<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Datenstruktur für Bäume.
 *
 * @package GhCore
 * @subpackage tree
 * @category Hilfsklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhCoreTree extends GhCoreTreeBase
{
  /**
   * Baum-Knoten nach ID (Schlüssel: IDs, Werte: Knoten).
   * @var array 
   */
  protected $_nodesById = array();

  /**
   * Klassenname für Baum-Knoten.
   * Neue Knoten werden als Instanzen dieser Klasse erzeugt.
   * @var string
   */
  protected $_treeNodeClass;

  /**
   * Konstruktor.
   *
   * @param string $treeNodeClass Klasse für Baum-Knoten
   */
  public function __construct($treeNodeClass = 'GhCoreTreeNode')
  {
    $this->_treeNodeClass = $treeNodeClass;
  }

  /**
   * Prüft, ob ein neuer Knoten zu einem gegebenen Baum-Knoten hinzugefügt werden kann.
   * Diese Methode kann überladen werden um das Hinzufügen bestimmter Daten zu unterbinden.
   *
   * @param int|string     $id      ID für den neuen Knoten
   * @param GhCoreTreeNode $parent  Eltern-Knoten, zu dem die Daten als neuer Kind-Knoten hinzugefügt werden sollen
   * @param string         $caption Titel für den neuen Kind-Knoten
   * @param mixed          $data    Daten für den neuen Kind-Knoten
   * @return boolean true falls der Knoten hinzugefügt werden kann, false falls nicht
   */
  protected function _canAddChild($id, $parent, $caption, $data)
  {
    return true;
  }

  /**
   * Fügt einem Baum-Knoten einen neuen Knoten hinzu.
   *
   * @param int|string     $id       ID für den neuen Knoten
   * @param GhCoreTreeBase $parent   Eltern-Knoten
   * @param string         $caption  Titel
   * @param mixed          $data     Daten
   * @param boolean        $prepend  true: der neue Knoten wird vorangestellt, false: der neue Knoten wird angehängt
   * @return GhCoreTreeNode Der neu hinzugefügte Knoten
   */
  protected function _addChild($id, GhCoreTreeBase $parent, $caption, $data, $prepend = false)
  {
    if($this->_canAddChild($id, $parent, $caption, $data))
    {
      $child = new $this->_treeNodeClass($this, $id, $parent, $caption, $data);
      $this->_nodesById[$id] = $child;

      if($prepend)
      {
        if($parent != $this)
          array_unshift($parent->_children, $child);
        else
          array_unshift($this->_children, $child);
      }
      else
      {
        if($parent != $this)
          $parent->_children[] = $child;
        else
          $this->_children[] = $child;
      }

      return $child;
    }
  }

  /**
   * Fügt einem Baum-Knoten einen neuen Knoten hinzu.
   *
   * @throws GhCoreException falls eine ungültige Eltern-Knoten ID übergeben wurde
   *
   * @param int|string $id       ID für den neuen Knoten
   * @param int|string $parentId ID des Eltern-Knotens. Falls eine 0 übergeben wird, dann wird der neue Knoten als Wurzelknoten hinzugefügt.
   * @param string     $caption  Titel des neuen Knotens
   * @param mixed      $data     Daten des neuen Knotens
   * @param boolean    $prepend  true: der neue Knoten wird vorangestellt, false: der neue Knoten wird angehängt
   * @return GhCoreTreeNode
   */
  public function addChild($id, $parentId, $caption, $data = null, $prepend = false)
  {
    if($parentId && isset($this->_nodesById[$parentId]))
      return $this->_addChild($id, $this->_nodesById[$parentId], $caption, $data, $prepend);
    else if(!$parentId)
      return $this->_addChild($id, $this, $caption, $data, $prepend);
    else
      throw new GhCoreException('Invalid parent id "' . $parentId . '" for new tree node');
  }

  /**
   * Fügt einen neuen Knoten als Wurzel hinzu.
   *
   * @param int    $id      ID des neuen Knotens
   * @param string $caption Titel des neuen Knotens
   * @param mixed  $data    Daten des neuen Knotens
   * @return GhCoreTreeNode Der neu erzeugte Wurzel-Knoten
   */
  public function add($id, $caption, $data = null)
  {
    return $this->addChild($id, null, $caption, $data);
  }

  /**
   * Liefert einen Knoten anhand seiner ID.
   *
   * @param int $id ID des Knotens
   * @return GhCoreTreeNode Der Knoten, oder false falls es keinen Knoten mit dieser ID gibt.
   */
  public function get($id)
  {
    if(isset($this->_nodesById[$id]))
      return $this->_nodesById[$id];
    else
      return false;
  }

  /**
   * Liefert die IDs aller Knoten des Baumes.
   *
   * @return array Alle Knoten-IDs
   */
  public function getIds()
  {
    return array_keys($this->_nodesById);
  }


  /**
   * Hilfsfunktion für getList().
   * 
   * @param GhCoreTreeBase $treeNode Knoten
   * @return array
   */
  protected function _getChildrenList(GhCoreTreeBase $treeNode)
  {
    $result = array();
    $children = $treeNode->getChildren();

    for($i = 0, $n = sizeof($children); $i < $n; $i++)
    {
      $result[] = $children[$i];
      $result = array_merge($result, $this->_getChildrenList($children[$i]));
    }

    return $result;
  }

  /**
   * Liefert eine Liste aller Baum-Knoten.
   *
   * @return array Liste aller Baum-Knoten
   */
  public function getList()
  {
    return $this->_getChildrenList($this);
  }

  /**
   * Hilfsfunktion für das Updaten der Zuordnung ID -> Tree-Knoten
   *
   * @param GhCoreTreeBase $treeNode Knoten
   * @param boolean        $reset    true: Zuordnungsliste zurücksetzen
   * @return array
   */
  protected function _updateNodesById(GhCoreTreeBase $treeNode, $reset = true)
  {
    if($reset)
      $this->_treeNodeClass = array();
      
    $children = $treeNode->getChildren();

    for($i = 0, $n = sizeof($children); $i < $n; $i++)
    {
      $this->_treeNodeClass[$children[$i]->getId()] = &$children[$i];
      $this->_updateNodesById($children[$i], false);
    }
  }

  /**
   * Setzt die Basis des Baums auf die angegebene ID.
   *
   * @param int $id ID des neuen Wurzelknotens
   * @return void
   */
  public function setRootId($id)
  {
    $treeNode = $this->get($id);

    if($treeNode)
    {
      $this->_children = $treeNode->getChildren();

      for($i = 0, $n = sizeof($this->_children); $i < $n; $i++)
        $this->_children[$i]->setParent($this);
        
      $this->_updateNodesById($this);
    }
  }
}