<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Button Komponente (Schaltfläche).
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiButton extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'button';

  /**
   * Attribute für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_attributes = array('type' => 'button');

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.Button';

  /**
   * Einstellung, ob der aktuelle POST/GET Wert für den Wert des Formularfelds
   * verwendet werden soll.
   */
  protected $_useCurrentValue = false;

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onClick');

  /**
   * Tastatur-Kommandos.
   * Die Elemente dieses Arrays sind Arrays deren erstes Element das Zeichen
   * oder der Tastencode und das zweite Element die Zusatztasten sind.
   * @var array
   */
  protected $_keyboardShortcuts = array();

  /**
   * Konstruktor.
   *
   * Optional kann die Beschriftung des Buttons angegeben werden, sowie die
   * auto-submit Funktion deaktiviert werden (siehe setAutoSubmit()). Bei
   * Buttons ist die auto-submit Funktion als Vorgabe eingeschaltet, so dass
   * ein Klick auf ein Button zum Absenden des Formulars führt.
   *
   * @param string  $caption    Beschriftung des Buttons
   * @param boolean $autoSubmit true: auto-submit, false: kein auto-submit
   */
  public function __construct($caption = null, $autoSubmit = true)
  {
    parent::__construct();
    $this->addClass('GhGuiButton');

    if($caption !== null)
      $this->setCaption($caption);

    $this->setAutoSubmit($autoSubmit);
  }

  /**
   * Setzt die Beschriftung des Buttons.
   *
   * @param string $caption Beschriftung
   * @return GhGuiButton liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaption($caption)
  {
    $this->setWidgetProperty('label', $caption);

    return $this;
  }

  /**
   * Liefert die Beschriftung des Buttons.
   *
   * @return string
   */
  public function getCaption()
  {
    return $this->getWidgetProperty('label');
  }

  /**
   * Fügt ein Tastatur-Kommando hinzu, welches diesen Button auslöst.
   *
   * @param string|int $key       Zeichen oder Tastencode
   * @param int        $modifiers Zusatztasten
   * @return GhGuiButton liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addKeyboardShortcut($key, $modifiers = null)
  {
    $this->_keyboardShortcuts[] = array($key, $modifiers);

    return $this;
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Falls das Button einen Feldnamen erhalten hat registriert es ein
   * verstecktes Formularfeld mit diesem Namen bei seinem Formular-Container,
   * damit beim Absenden des Formulars der Name und Wert des Buttons mit
   * gesendet werden.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $name = $this->getName();

    if($name !== null && $this->getAutoSubmit())
    {
      $form = $this->getForm();

      if(!$form)
        throw new GhGuiException('The GhGuiButton has not been added to a GhGuiForm');

      $value = $this->getValue();

      if(!$value && $value !== '0')
        $value = $this->getCaption();

      // Event-Handler bei Klick: hidden field mit Name und Wert zum Formular hinzufügen:
      $this->addEventHandler('onClick', new GhGuiEventJavascript('function(event){dojo.create("input",{type:"hidden",' . GhCoreStrings::jsonProperty('name', $name, 'string') . ',' . GhCoreStrings::jsonProperty('value', $value, 'string') . '},dojo.byId("' . $form->getHiddenFieldsContainer()->getId() . '"));}'));
    }

    if(!empty($this->_keyboardShortcuts))
    {
      $page = $this->getPage();

      foreach($this->_keyboardShortcuts as $shortcut)
        $page->addKeyboardShortcut($shortcut[0], $shortcut[1], new GhGuiEventJavascript('function(event){' . $this->getJavascriptWidgetVariable() . '.onClick(event);}'));
    }

    return true;
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    return GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes(), $this->getCaption());
  }
}