<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Combo-Box Komponente (Textfeld mit Drop-Down Liste).
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiComboBox extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'select';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.ComboBox';

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange');

  /**
   * Einträge der Drop-Down Liste.
   * @var array
   */
  protected $_items = array();

  /**
   * Eigenschaft der Einträge, in der die Beschriftung für die Drop-Down Liste
   * enthalten ist.
   * @var string
   */
  protected $_captionProperty = 'Name';

  /**
   * Eigenschaft der Einträge, in der der Wert für die Drop-Down Liste
   * enthalten ist.
   * @var string
   */
  protected $_valueProperty = 'ID';


  /**
   * Konstruktor.
   *
   * Optional kann der Name des Formularfeldes und ein Array mit Einträgen für
   * die Drop-Down Liste angegeben werden.
   *
   * @param string $name  Name des Formularfeldes
   * @param array  $items Einträge für die Drop-Down Liste
   */
  public function __construct($name = null, array $items = null)
  {
    parent::__construct($name);
    $this->addClass('GhGuiComboBox');

    if($items !== null)
      $this->addItems($items);
  }

  /**
   * Fügt einen Eintrag der Drop-Down Liste hinzu.
   *
   * Der Eintrag muss entweder ein Array oder ein Objekt sein.
   *
   * @param object|array $item der hinzuzufügende Eintrag
   * @return GhGuiComboBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addItem($item)
  {
    if(!is_object($item) && !is_array($item))
      throw new GhGuiException('Only arrays and objects may be added as items to a GhGuiComboBox');

    $this->_items[] = $item;

    return $this;
  }

  /**
   * Fügt mehrere Einträge der Drop-Down Liste hinzu.
   *
   * Die Einträge müssen entweder Arrays oder Objekte sein.
   *
   * @param array $items die hinzuzufügenden Einträge
   * @return GhGuiComboBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addItems(array $items)
  {
    foreach ($items as $item)
      $this->addItem($item);

    return $this;
  }

  /**
   * Liefert die Einträge der Drop-Down liste.
   *
   * @return array
   */
  public function getItems()
  {
    return $this->_items;
  }

  /**
   * Setzt die Objekteigenschaft der Datenobjekte für den Wert des Formularfeldes.
   *
   * @param string $property Objekteigenschaft
   * @return GhGuiComboBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValueProperty($property)
  {
    $this->_valueProperty = $property;

    return $this;
  }

  /**
   * Liefert die Objekteigenschaft der Datenobjekte bzw. den Schlüssel der
   * Daten-Arrays für den Wert des Formularfeldes.
   *
   * @return string
   */
  public function getValueProperty()
  {
    return $this->_valueProperty;
  }

  /**
   * Setzt die Objekteigenschaft der Datenobjekte für die Beschriftung der Listeneinträge.
   *
   * @param string $property Objekteigenschaft
   * @return GhGuiComboBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaptionProperty($property)
  {
    $this->_captionProperty = $property;

    return $this;
  }

  /**
   * Liefert die Objekteigenschaft der Datenobjekte bzw. den Schlüssel der
   * Daten-Arrays für die Beschriftung der Listeneinträge.
   *
   * @return string
   */
  public function getCaptionProperty()
  {
    return $this->_captionProperty;
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    $value = $this->getCurrentValue();

    $options = '';
    $captionProperty = $this->_captionProperty;
    $valueProperty = $this->_valueProperty;
    foreach ($this->_items as $item)
    {
      if(is_object($item))
      {
        $itemCaption = $item->$captionProperty;
        $itemValue = $item->$valueProperty;
      }
      else
      {
        $itemCaption = $item[$captionProperty];
        $itemValue = $item[$valueProperty];
      }
      $optionAttributes = array('value' => $itemValue);

      if ($itemValue == $value)
        $optionAttributes['selected'] = 'selected';

      $options .= GhGui::createHtml('option', $optionAttributes, $itemCaption);
    }

    return GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes(), $options);
  }
}