<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Combo-Tree (Textfeld mit Drop-Down Baum).
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiComboTree extends GhGuiTree
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.ComboTree';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dojo.data.ObjectStore', 'dojo.store.Memory', 'dijit.tree.ForestStoreModel', 'dijit.Tree', 'dijit.form.ComboBox');

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange', 'onSelected');

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiComboTree');

  /**
   * Konstruktor.
   *
   * Optional können der Name des Formularfeldes und der Baum für die Auswahl
   * angegeben werden.
   * Zudem kann die Objekteigenschaft bzw. der Array-Schlüssel angegeben
   * werden, über die bei Auswahl eines Knotens aus den Daten der Baumknoten
   * der Wert für das Formularfeld bezogen wird.
   *
   * Wenn es sich bei den Daten in den Baumknoten um Objekte handelt, dann wird
   * der Wert für das Formularfeld aus der angegebenen Objekteigenschaft
   * bezogen wenn ein Baumknoten ausgewählt wird. Handelt es sich bei den Daten
   * um Arrays, so wird die Eigenschaft als Schlüssel verwendet.
   *
   * Wird false übergeben, so werden die Daten der Baumknoten direkt als Wert
   * für das Formularfeld verwendet, ohne sie als Objekt oder Array zu
   * interpretieren. Die Daten müssen in diesem Fall skalar sein (Strings oder
   * Zahlen).
   *
   * @param string     $name Name des Formularfeldes
   * @param GhCoreTree $tree Baum mit den Daten für den Drop-Down Baum
   * @param string $valueProperty Objekteigenschaft
   */
  public function __construct($name = null, GhCoreTree $tree = null, $valueProperty = null)
  {
    parent::__construct($name, $tree, $valueProperty);
  }

  /**
   * Liefert den Javascript Code für diese Komponente.
   *
   * @return string
   */
  protected function _getJavascript()
  {
    $javascript = 'var store_' . $this->getId() . '=new dojo.data.ObjectStore({objectStore:new dojo.store.Memory({data:[';

    if($this->_tree instanceof GhCoreTree)
      $javascript .= $this->_getTreeJson($this->_tree);

    $javascript .= ']})});' . GhGui::$eol;
    $javascript .= 'var model_' . $this->getId() . '=new dijit.tree.ForestStoreModel({store:store_' . $this->getId() . ',rootId:"' . GhGui::createId() . '",rootLabel:"#"});' . GhGui::$eol;

    $properties = $this->_getWidgetCreationProperties();
    $properties['model'] = 'model_' . $this->getId();
    $properties['showRoot'] = false;
    $properties['persist'] = false;

    if($this->getName() !== null)
    {
      $properties['updateValueNode'] = 'dojo.byId("' . $this->getValueNodeId() . '")';
      $properties['updateValueItemProperty'] = 'value';
    }

    // Icons:
    if($this->_iconProperty !== null || $this->_iconOpenProperty !== null || $this->_iconNode !== null || $this->_iconNodeOpen !== null || $this->_iconLeaf !== null)
    {
      $iconSet = $this->getIconSet(true);

      if($iconSet)
      {
        $iconClass = $iconSet->getIconSetClass();
        $iconsJavascript = 'function(item,opened){' . GhGui::$eol;

        // Icons in Abhängigkeit von Dateneigenschaften:
        if($this->_iconProperty !== null && $this->_iconOpenProperty !== null)
          $iconsJavascript .= 'if(item.iconOpen && opened)return "' . $iconClass . ' "+item.iconOpen;else if(item.icon)return "' . $iconClass . ' "+item.icon;' . GhGui::$eol;
        elseif($this->_iconProperty !== null && $this->_iconOpenProperty === null)
          $iconsJavascript .= 'if(item.icon)return "' . $iconClass . ' "+item.icon;' . GhGui::$eol;

        // Standard-Icons dieses Baums:
        if($this->_iconLeaf !== null)
          $iconLeaf = $iconClass . ' ' . $this->_iconLeaf;
        else
          $iconLeaf = 'dijitLeaf';

        if($this->_iconNode !== null)
          $iconNode = $iconClass . ' ' . $this->_iconNode;
        else
          $iconNode = 'dijitFolderClosed';

        if($this->_iconNodeOpen !== null)
          $iconNodeOpen = $iconClass . ' ' . $this->_iconNodeOpen;
        else
          $iconNodeOpen = 'dijitFolderOpened';

        $iconsJavascript .= 'if(!item.children || item.children.length==0)return "' . $iconLeaf . '";else if(opened)return "' . $iconNodeOpen . '";else return "' . $iconNode . '";' . GhGui::$eol;

        $iconsJavascript .= '}' . GhGui::$eol;
        $properties['getIconClass'] = $iconsJavascript;
      }
    }

    $javascript .= $this->_getWidgetCreationJavascript($this->_widgetClass, $properties);

    // Vorausgewähltes Element selektieren:
    $value = $this->getCurrentValue();

    if($this->getName() !== null && $value !== null)
      $javascript .= 'var treePath=greyhound.dojo.getTreePath(' . $this->getJavascriptWidgetVariable() . ',"value","' . str_replace('"', '\\"', $value) . '");if(treePath)' . $this->getJavascriptWidgetVariable() . '.set("path",treePath);' . GhGui::$eol;

    return $javascript;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'updateValueItemProperty':
        return 'string';

      case 'updateValueNode':
        return 'variable';
    }

    return parent::__getWidgetPropertyType($property);
  }

  /**
   * Registriert den Event-Handler für Events im Tree.
   */
  protected function _initHtmlEvents()
  {
    // Der ComboTree arbeitet mit Selection-Events. Das Click-Event auf der Textbox darf nicht wie im GhGuiTree verarbeitet werden, sonst geht der ausgewählte Wert verloren (id#2202)
  }
}