<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Dateiauswahl-Button.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiFileChooser extends GhGuiFormComponent
{
  /**
   * Nur eine Datei auswählbar.
   * @var const
   */
  const FILES_SINGLE = 1;

  /**
   * Beliebig viele Dateien auswählbar.
   * @var const
   */
  const FILES_MULTIPLE = 0;

  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'input';

  /**
   * Attribute für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_attributes = array('type' => 'file');

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiFileChooser');

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.FileChooser';

  /**
   * Einstellung, ob der aktuelle POST/GET Wert für den Wert des Formularfelds
   * verwendet werden soll.
   */
  protected $_useCurrentValue = false;

  /**
   * Konstruktor.
   *
   * @param string $name       Name des Formularfelds
   * @param int    $maxNrFiles maximale Anzahl auswählbarer Dateien (Wert 0
   *   bedeutet: beliebig viele, der Vorgabewert ist 1 (nur eine Datei))
   */
  public function __construct($name = null, $maxNrFiles = null)
  {
    parent::__construct($name);

    if($maxNrFiles === null)
      $maxNrFiles = GhGuiFileChooser::FILES_SINGLE;

    $this->setMaxNrFiles($maxNrFiles);
  }

  /**
   * Setzt die maximale Anzahl an Dateien, die mit dieser Komponente ausgewählt
   * und hochgeladen werden können.
   *
   * @param int $maxNrFiles maximale Anzahl auswählbarer Dateien (Wert 0
   *   bedeutet: beliebig viele, der Vorgabewert ist 1 (nur eine Datei))
   * @return GhGuiFileChooser liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setMaxNrFiles($maxNrFiles)
  {
    $this->setWidgetProperty('maxNrFiles', (int)$maxNrFiles);

    return $this;
  }

  /**
   * Liefert die maximale Anzahl an Dateien, die mit dieser Komponente
   * ausgewählt werden können.
   *
   * @return int
   */
  public function getMaxNrFiles()
  {
    return $this->getWidgetProperty('maxNrFiles');
  }

  /**
   * Legt die Beschriftung der Komponente fest.
   *
   * Es kann jeweils eine Beschriftung dafür angegeben werden, dass noch keine
   * Dateien ausgewählt wurden, dass eine einzelne Datei ausgewählt wurde, und
   * dass mehrere Dateien ausgewählt wurden.
   *
   * In der Beschriftung können die Zeichenketten <tt>#NRFILES#</tt> für die
   * Anzahl der ausgewählten Dateien, und <tt>#FILENAMES#</tt> für die Namen
   * der ausgewählten Dateien verwendet werden. Diese werden in der
   * Beschriftung entsprechend den ausgewählten Dateien ersetzt.
   *
   * Wird keine Beschriftung für mehrere Dateien angegeben, so wird die
   * Beschriftung für eine Datei verwendet. Wird keine Beschriftung für eine
   * Datei angegeben, so wird die Beschriftung für keine Dateien verwendet.
   *
   * @param string $noFiles       Beschriftung wenn keine Datei gewählt wurde
   * @param string $singleFile    Beschriftung bei Auswahl einer Datei
   * @param string $multipleFiles Beschriftung bei Auswahl mehrerer Dateien
   */
  public function setCaption($noFiles, $singleFile = null, $multipleFiles = null)
  {
    $this->setWidgetProperty('captionNoFiles', $noFiles);
    $this->setWidgetProperty('captionSingleFile', $singleFile);
    $this->setWidgetProperty('captionMultipleFiles', $multipleFiles);
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    $form = $this->getForm();

    if(!$form)
      throw new GhGuiException('The GhGuiFileChooser has not been added to a GhGuiForm');

    // Bei Datei-Uploads multipart-Formular verwenden:
    $form->setEncodingType('multipart/form-data');

    $this->setWidgetProperty('formId', $form->getId());

    // Bei Mehrfachauswahl sicherstellen, dass der Komponentenname einen Array erzeugt:
    if($this->getMaxNrFiles() != 1)
    {
      $name = $this->getName();
      if(strlen($name) >= 2 && substr($name, strlen($name) - 2) != '[]')
      {
        $name .= '[]';
        $this->setName($name);
      }
    }

    return true;
  }
}