<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Hyperlink-Komponente.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiLink extends GhGuiComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'a';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.Link';

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiLink');

  /**
   * Beschriftung des Links.
   * @var string
   */
  protected $_caption;

  /**
   * Konstruktor.
   *
   * @param string $url     Ziel-URL
   * @param string $caption Beschriftung
   * @param string $icon    Icon-Name
   * @param string $target  Link-Ziel
   */
  public function __construct($url = null, $caption = null, $icon = null, $target = null)
  {
    parent::__construct();

    if($url !== null)
      $this->setUrl($url);

    if($caption !== null)
      $this->setCaption($caption);

    if($icon !== null)
      $this->setIcon($icon);

    if($target !== null)
      $this->setTarget($target);
  }

  /**
   * Setzt die Ziel-URL des Links.
   *
   * @param string $url Ziel-URL
   * @return GhGuiLink liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setUrl($url)
  {
    $this->setAttribute('href', $url);

    return $this;
  }

  /**
   * Liefert die Ziel-URL des Links.
   *
   * @return string
   */
  public function getUrl()
  {
    return $this->getAttribute('href');
  }

  /**
   * Setzt die Beschriftung des Links.
   *
   * @param string $caption Beschriftung
   * @return GhGuiLink liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaption($caption)
  {
    $this->_caption = $caption;

    return $this;
  }

  /**
   * Liefert die Beschriftung des Links.
   *
   * @return string
   */
  public function getCaption()
  {
    return $this->_caption;
  }

  /**
   * Setzt das Zielfenster des Links.
   *
   * @param string $target Link-Ziel
   * @return GhGuiLink liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTarget($target)
  {
    $this->setAttribute('target', $target);

    return $this;
  }

  /**
   * Liefert das Zielfenster des Links.
   * @return string
   */
  public function getTarget()
  {
    return $this->getAttribute('target');
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    $html = '';

    $icon = $this->getIcon();

    if($icon)
    {
      $iconSet = $this->getIconSet(true);

      if($iconSet)
        $html .= GhGui::createHtml('img', array('src' => GhGui::createUrl('ghgui://lib/dojo/resources/blank.gif'), 'class' => $iconSet->getIconSetClass() . ' ' . $icon));
    }

    $caption = $this->getCaption();

    if($caption !== null)
      $html .= $caption;

    return GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes(), $html);
  }
}