<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Liste.
 * 
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiList extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'select';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'greyhound.dojo.List';

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiList');

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange');

  /**
   * Einträge der Liste.
   * @var array
   */
  protected $_items = array();

  /**
   * Eigenschaft der Einträge, in der die Beschriftung für die Liste enthalten
   * ist.
   * @var string
   */
  protected $_captionProperty = 'Name';

  /**
   * Eigenschaft der Einträge, in der der Wert für die Liste enthalten ist.
   * @var string
   */
  protected $_valueProperty = 'ID';

  /**
   * Konstruktor.
   *
   * Optional kann der Name des Formularfeldes und ein Array mit Einträgen für
   * die Liste angegeben werden, sowie festgelegt werden, ob Mehrfachauswahl
   * möglich sein soll (Voreinstellung: nur Einfachauswahl).
   *
   * @param string  $name        Name des Formularfeldes
   * @param array   $items       Einträge für die Liste
   */
  public function __construct($name = null, $rows = null, $multiple = false, array $items = null)
  {
    parent::__construct($name);

    if($rows === null)
      $rows = 4;

    $this->setRows($rows);

    $this->setMultipleSelection($multiple);

    if($items !== null)
      $this->addItems($items);
  }

  /**
   * Fügt einen Eintrag der Drop-Down Liste hinzu.
   *
   * Der Eintrag muss entweder ein Array oder ein Objekt sein.
   *
   * @param object|array $item der hinzuzufügende Eintrag
   * @return GhGuiList liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addItem($item)
  {
    if(!is_object($item) && !is_array($item))
      throw new GhGuiException('Only arrays and objects may be added as items to a GhGuiComboBox');

    $this->_items[] = $item;

    return $this;
  }

  /**
   * Fügt mehrere Einträge der Drop-Down Liste hinzu.
   *
   * Die Einträge müssen entweder Arrays oder Objekte sein.
   *
   * @param array $items die hinzuzufügenden Einträge
   * @return GhGuiList liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addItems(array $items)
  {
    foreach ($items as $item)
      $this->addItem($item);

    return $this;
  }

  /**
   * Liefert die Einträge der Drop-Down liste.
   *
   * @return array
   */
  public function getItems()
  {
    return $this->_items;
  }

  /**
   * Setzt die Anzahl der sichtbaren Zeilen der Liste.
   *
   * @param int $rows Anzahl sichtbarer Zeilen
   * @return GhGuiList liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setRows($rows)
  {
    $this->setWidgetProperty('size', (int)$rows);

    return $this;
  }

  /**
   * Liefert die Anzahl der sichtbaren Zeilen der Liste.
   *
   * @return int
   */
  public function getRows()
  {
    return $this->getWidgetProperty('size');
  }

  /**
   * Legt fest, ob mehrere Elemente ausgewählt werden können (Mehrfachauswahl),
   * oder nur eines (Einfachauswahl).
   *
   * @param boolean $multiple true: Mehrfachauswahl, false: Einfachauswahl
   * @return GhGuiList liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setMultipleSelection($multiple)
  {
    $this->setWidgetProperty('multiple', $multiple ? true : false);

    return $this;
  }

  /**
   * Liefert die Einstellung für Mehrfach-/Einfachauswahl.
   *
   * @return boolean
   */
  public function getMulipleSelection()
  {
    return $this->getWidgetProperty('multiple');
  }

  /**
   * Setzt die Dateneigenschaft, die den Wert für das Formularfeld enthält.
   *
   * @param string $property Eigenschaft der Datenobjekte/-arrays
   * @return GhGuiDataGrid liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValueProperty($property)
  {
    $this->_valueProperty = $property;

    return $this;
  }

  /**
   * Liefert die Datenteigenschaft, die den Wert für das Formularfeld enthält.
   *
   * @return string Datenobjekteigenschaft
   */
  public function getValueProperty()
  {
    return $this->_valueProperty;
  }

  /**
   * Setzt die Dateneigenschaft, die die Beschriftung für den Listeneintrag
   * enthält.
   *
   * @param string $property Eigenschaft der Datenobjekte/-arrays
   * @return GhGuiDataGrid liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaptionProperty($property)
  {
    $this->_captionProperty = $property;

    return $this;
  }
  
  /**
   * Liefert die Datenteigenschaft, die die Beschriftung für den Listeneintrag
   * enthält.
   *
   * @return string Datenobjekteigenschaft
   */
  public function getCaptionProperty()
  {
    return $this->_captionProperty;
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    $attributes = $this->_getAllAttributes();
    $attributes['size'] = $this->getRows();

    if($this->getMulipleSelection())
    {
      $attributes['multiple'] = 'multiple';

      // Sicherstellen, dass der Name auf '[]' endet, da sonst nur ein
      // ausgewähltes Element abgeschickt wird:
      if(isset($attributes['name']))
        $attributes['name'] = rtrim($attributes['name'], '[]') . '[]';
    }

    return GhGui::createHtml($this->_htmlTag, $attributes, $this->_getItemsHtml());
  }

  /**
   * Liefert den HTML-Code für die Listeneinträge ('option'-Knoten).
   *
   * @return string
   */
  protected function _getItemsHtml()
  {
    $html = '';

    $value = $this->getCurrentValue();
    // Wert als Array, um Mehrfachauswahl zu unterstützen:
    if(!is_array($value))
      $value = array($value);
    $captionProperty = $this->_captionProperty;
    $valueProperty = $this->_valueProperty;
    foreach ($this->getItems() as $item)
    {
      if(is_object($item))
      {
        $itemCaption = isset($item->$captionProperty)
          ? $item->$captionProperty
          : '';
        $itemValue = isset($item->$valueProperty)
          ? $item->$valueProperty
          : '';
      }
      else
      {
        $itemCaption = isset($item[$captionProperty])
          ? $item[$captionProperty]
          : '';
        $itemValue = isset($item[$valueProperty])
          ? $item[$valueProperty]
          : '';
      }
      $optionAttributes = array('value' => $itemValue);

      if (in_array($itemValue, $value))
        $optionAttributes['selected'] = 'selected';

      $html .= GhGui::createHtml('option', $optionAttributes, $itemCaption);
    }

    return $html;
  }
}