<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Text Komponente.
 *
 * Diese Komponente stellt beliebigen Text dar. Der Text kann durch hinzufügen
 * von Überschriften und Textinhalten, aber auch durch einfügen anderer
 * Komponenten zusammengestellt werden.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiText extends GhGuiComponent
{
  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.layout.ContentPane';

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiText');

  /**
   * Inhalte.
   * @var array
   */
  protected $_pieces = array();

  /**
   * Legt fest, ob URLs der Form ghgui://... durch die Basis-URL der Webseite
   * ersetzt werden.
   * @var boolean
   */
  protected $_replaceUrls = true;

  /**
   * Konstruktor.
   *
   * Optional kann Text (oder HTML Code) übergeben werden, der dargestellt
   * werden soll.
   *
   * @param string  $text      Text
   * @param boolean $quoteHtml HTML Sonderzeichen maskieren (htmlspecialchars)
   */
  public function __construct($text = null, $quoteHtml = false)
  {
    parent::__construct();

    if($text !== null)
      $this->addText($text, $quoteHtml);
  }

  /**
   * Leert den Text.
   */
  public function clear()
  {
    $this->_pieces = array();
  }

  /**
   * Legt fest, ob URLs im Format ghgui:// durch die Basis-URL der Webseite
   * ersetzt werden.
   *
   * Die Voreinstellung ist, dass solche URLs ersetzt werden.
   *
   * @param boolean $replaceUrls true: URLs ersetzen, false: URLs nicht ersetzen
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setReplaceUrls($replaceUrls)
  {
    $this->_replaceUrls = $replaceUrls;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob URLs im Format ghgui:// durch die Basis-URL
   * der Webseite ersetzt werden.
   *
   * @return boolean
   */
  public function getReplaceUrls()
  {
    return $this->_replaceUrls;
  }

  /**
   * Fügt Text hinzu.
   *
   * @param string  $text       Text
   * @param boolean $quoteHtml  HTML Sonderzeichen maskieren (htmlspecialchars)
   * @param boolean $lineBreaks Zeilenumbrüche im Text in HTML-Zeilenumbrüche umwandeln
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addText($text, $quoteHtml = false, $lineBreaks = false)
  {
    if($quoteHtml)
      $text = htmlspecialchars($text, ENT_NOQUOTES, GhGui::getCharset());

    if($lineBreaks)
      $text = nl2br($text, GhGui::isXml());

    $this->_pieces[] = array('type' => 'text', 'value' => $text);

    return $this;
  }

  /**
   * Fügt eine Überschrift hinzu.
   *
   * @param string $heading Text der Überschrift
   * @param int    $level   Gliederungsebene
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addHeading($heading, $level = null)
  {
    if(!is_numeric($level))
      $level = 1;

    $this->_pieces[] = array('type' => 'text', 'value' => GhGui::createHtml('h' . $level, null, htmlspecialchars($heading, ENT_NOQUOTES, GhGui::getCharset())));

    return $this;
  }

  /**
   * Fügt einen Absatz Text hinzu.
   *
   * @param string $text Text
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addParagraph($text)
  {
    $this->_pieces[] = array('type' => 'text', 'value' => GhGui::createHtml('p', null, htmlspecialchars($text, ENT_NOQUOTES, GhGui::getCharset())));

    return $this;
  }

  /**
   * Fügt einen Zeilenumbruch hinzu.
   *
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addLineBreak()
  {
    $this->_pieces[] = array('type' => 'text', 'value' => GhGui::createHtml('br'));

    return $this;
  }

  /**
   * Fügt eine horizontale Linie hinzu.
   *
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addHorizontalLine()
  {
    $this->_pieces[] = array('type' => 'text', 'value' => GhGui::createHtml('hr'));

    return $this;
  }

  /**
   * Fügt ein Bild hinzu.
   *
   * @param string $url        URL des Bilds
   * @param array  $attributes Attribute des HTML-Knotens des Bilds
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addImage($url, array $attributes = null)
  {
    $attributes['src'] = $url;
    $this->_pieces[] = array('type' => 'text', 'value' => GhGui::createHtml('img', $attributes));

    return $this;
  }

  /**
   * Fügt ein Icon hinzu.
   *
   * @param string $icon Name des Icons
   * @param array  $attributes Attribute des HTML-Knotens des Icon-Bilds
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addIcon($icon, array $attributes = null)
  {
    $this->_pieces[] = array('type' => 'icon', 'value' => $icon, 'attributes' => $attributes);

    return $this;
  }

  /**
   * Fügt den Inhalt des 'body'-Knotens einer HTML-Seite zu dieser Komponente
   * hinzu.
   *
   * @param string $html HTML Text
   * @return GhGuiText liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addHtmlBody($html)
  {
    return $this->addText(GhCoreStrings::getHtmlBody($html));
  }

  /**
   * Liefert den HTML Code für diese Komponente.
   *
   * @return string
   */
  protected function _getHtml()
  {
    $html = '';

    foreach($this->_pieces as $piece)
    {
      switch($piece['type'])
      {
        case 'text':
          $html .= $piece['value'];
          break;

        case 'icon':
          if(!isset($iconSet))
          {
            $iconSet = $this->getIconSet(true);

            if($iconSet)
              $defaultSize = $iconSet->getDefaultIconSize();

            if(!isset($defaultSize) || !is_array($defaultSize) || count($defaultSize) < 2)
              $defaultSize = array(16, 16);
          }
          else
            $defaultSize = array(16, 16);

          if($iconSet)
          {
            if(is_array($piece['attributes']))
              $attributes = $piece['attributes'];
            else
              $attributes = array();

            $attributes['src'] = GhGui::createUrl('ghgui://lib/dojo/resources/blank.gif');
            $attributes['class'] = $iconSet->getIconSetClass() . ' ' . $piece['value'];

            $width = $defaultSize[0];
            $height = $defaultSize[1];
            $icon = $iconSet->getIcon($piece['value']);

            if(is_array($icon) && count($icon) > 2)
            {
              $width = $icon[2];

              if(isset($icon[3]))
                $height = $icon[3];
            }

            $attributes['style'] = 'width:' . $width . 'px;height:' . $height . 'px;';
            $html .= GhGui::createHtml('img', $attributes);
          }
          break;
      }
    }

    if($this->getReplaceUrls())
      $html = str_replace('ghgui://', GhGui::getBaseUrl(), $html);

    return GhGui::createHtml($this->_htmlTag, $this->_getAllAttributes(), $html);
  }
}
