<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Textfeld Komponente (Text-Box).
 *
 * Dies ist ein einzeiliges Textfeld.
 * Passwort-Eingabefelder können durch die [classes]GhGuiPasswordBox[/classes]
 * Komponente, und mehrzeilige Textbereiche durch die
 * [classes]GhGuiTextArea[/classes] Komponente dargestellt werden.
 *
 * @package GhGui
 * @subpackage component
 * @category PHP-GUI-Einzelkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTextBox extends GhGuiFormComponent
{
  /**
   * Name/Typ des HTML-Knotens der für diese Komponente erzeugt wird.
   * @var string
   */
  protected $_htmlTag = 'input';

  /**
   * Attribute für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_attributes = array('type' => 'text');

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.form.TextBox';

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array('onChange');

  /**
   * Legt fest, ob das Formular abgeschickt wird wenn die Enter-Taste in der
   * Text-Box gedrückt wird.
   * @var boolean
   */
  protected $_submitOnEnter = true;

  /**
   * Versteckte Formularfelder, die beim Drücken der Enter-Taste in der
   * Text-Box beim Absenden des Formulars hinzugefügt werden.
   * Die Einträge sind jeweils Arrays mit dem Namen des Formularfelds als
   * erstem Wert und dem Wert des Formularfelds als zweitem Wert.
   * @var array
   */
  protected $_submitOnEnterHiddenFields = array();

  /**
   * Konstruktor.
   *
   * Optional kann der Name des Eingabefeldes (siehe setName()) und der
   * Vorgabewert (siehe setDefaultValue()) festgelegt werden.
   *
   * @param string  $name         Name des Formularfeldes
   * @param string  $defaultValue Vorgabewert für das Formularfeld
   */
  public function __construct($name = null, $defaultValue = null)
  {
    parent::__construct($name, null, $defaultValue);
    $this->setWidgetProperty('type', 'text');
  }

  /**
   * Setzt einen Platzhalter-Text.
   *
   * Der Platzhalter wird angezeigt wenn die Text-Box keinen Text enthält und
   * verschwindet, sobald die Text-Box den Eingabefokus oder eine Eingabe
   * erhält.
   *
   * @param string $text der Platzhalter-Text
   * @return GhGuiTextBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setPlaceHolder($text)
  {
    $this->setWidgetProperty('placeHolder', $text);

    return $this;
  }

  /**
   * Liefert den Platzhalter-Text.
   *
   * @return string
   */
  public function getPlaceHolder()
  {
    return $this->getWidgetProperty('placeHolder');
  }

  /**
   * Legt fest, ob das Formular beim Drücken der Enter-Taste abgeschickt wird.
   *
   * @param boolean $submit true: Formular abschicken wenn die Enter-Taste
   *   gedrückt wird, false: Formular nicht abschicken
   * @return GhGuiTextBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setSubmitOnEnter($submit)
  {
    $this->_submitOnEnter = $submit ? true : false;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob das Formular beim Drücken der Enter-Taste in
   * der Text-Box abgeschickt wird.
   *
   * @return boolean
   */
  public function getSubmitOnEnter()
  {
    return $this->_submitOnEnter;
  }

  /**
   * Fügt dem Formular ein verstecktes Feld hinzu wenn es durch Drücken der
   * Enter-Taste in der Text-Box abgesendet wird.
   *
   * @param string $name  Name des versteckten Formularfelds
   * @param string $value Wert des versteckten Formularfelds
   * @return GhGuiTextBox liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addSubmitOnEnterHiddenField($name, $value)
  {
    $this->_submitOnEnterHiddenFields[] = array($name, $value);

    return $this;
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   *
   * Falls das Button einen Feldnamen erhalten hat registriert es ein
   * verstecktes Formularfeld mit diesem Namen bei seinem Formular-Container,
   * damit beim Absenden des Formulars der Name und Wert des Buttons mit
   * gesendet werden.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    if($this->getSubmitOnEnter())
    {
      $javascript = 'function(event){if(event.charOrCode==dojo.keys.ENTER){';
      $hiddenFieldsContainerId = $this->getForm()->getHiddenFieldsContainer()->getId();

      foreach($this->_submitOnEnterHiddenFields as $field)
        $javascript .= 'dojo.create("input",{' .
                       'type:"hidden",' .
                       GhCoreStrings::jsonProperty('name', $field[0], 'string') . ',' .
                       GhCoreStrings::jsonProperty('value', $field[1], 'string') .
                       '},dojo.byId("' . $hiddenFieldsContainerId . '"));';

      $javascript .= $this->getForm()->getJavascriptWidgetVariable() . '.submit();';
      $javascript .= '};}';

      $this->addEventHandler('onKeyPress', new GhGuiEventJavascript($javascript));
    }

    return true;
  }
}