<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Border-Layout GUI Container.
 *
 * Dieser Container stellt Komponenten in fünf Bereichen (Oben, Unten, Links,
 * Rechts und Mitte) dar.
 *
 * Der Container hat zwei mögliche Layouts:
 * <dl>
 * <dt><tt>GhGuiBorderContainer::LAYOUT_HEADLINE</tt></dt>
 * <dd>Der obere und der untere Bereich nehmen die gesamte Breite ein und
 * liegen über bzw. unter dem linken und rechten Bereich, sowie dem
 * Inhaltsbereich.</dd>
 * <dt><tt>GhGuiBorderContainer::LAYOUT_SIDEBAR</tt></dt>
 * <dd>Der linke und der rechte Bereich nehmen die gesamte Höhe ein. Der obere
 * und untere Bereich liegen zwischen dem linken und rechten Bereich.</dd>
 * </dl>
 * Die Voreinstellung ist <tt>GhGuiBorderContainer::LAYOUT_SIDEBAR</tt>.
 *
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiBorderContainer extends GhGuiContainer
{
  /**
   * Headline Layout.
   * Der obere und der untere Bereich nehmen die gesamte Breite ein und
   * liegen über bzw. unter dem linken und rechten Bereich, sowie dem
   * Inhaltsbereich.
   */
  const LAYOUT_HEADLINE = 'headline';

  /**
   * Sidebar Layout.
   * Der linke und der rechte Bereich nehmen die gesamte Höhe ein. Der obere
   * und untere Bereich liegen zwischen dem linken und rechten Bereich.
   */
  const LAYOUT_SIDEBAR = 'sidebar';

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.layout.BorderContainer';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * Komponenten des oberen Bereichs.
   * @var array
   */
  protected $_top = array();

  /**
   * Komponenten des rechten Bereichs.
   * @var array
   */
  protected $_right = array();

  /**
   * Komponenten des unteren Bereichs.
   * @var array
   */
  protected $_bottom = array();

  /**
   * Komponenten des linken Bereichs.
   * @var array
   */
  protected $_left = array();

  /**
   * Komponenten des mittleren Bereichs.
   * @var array
   */
  protected $_center = array();

  /**
   * Konstruktor.
   *
   * Optional kann der Layout-Modus des Containers festgelegt werden:
   * <dl>
   * <dt><tt>GhGuiBorderContainer::LAYOUT_HEADLINE</tt></dt>
   * <dd>Der obere und der untere Bereich nehmen die gesamte Breite ein und
   * liegen über bzw. unter dem linken und rechten Bereich, sowie dem
   * Inhaltsbereich.</dd>
   * <dt><tt>GhGuiBorderContainer::LAYOUT_SIDEBAR</tt></dt>
   * <dd>Der linke und der rechte Bereich nehmen die gesamte Höhe ein. Der obere
   * und untere Bereich liegen zwischen dem linken und rechten Bereich.</dd>
   * </dl>
   * Die Voreinstellung ist <tt>GhGuiBorderContainer::LAYOUT_SIDEBAR</tt>.
   *
   * Zudem kann festgelegt werden, ob Trennlinien angezeigt werden sollen, mit
   * denen (durch Ziehen) die Größe der Bereiche verändert werden kann.
   *
   * @param string $layoutMode Layout-Modus
   * @param boolean $splitters true: Trennlinien für Größenänderung anzeigen,
   *   false: keine Größenänderung ermöglichen
   */
  public function __construct($layoutMode = null, $splitters = null)
  {
    parent::__construct();
    $this->setWidgetProperty('gutters', false);

    if($layoutMode === null)
      $layoutMode = self::LAYOUT_SIDEBAR;
    $this->setLayoutMode($layoutMode);

    if($splitters === null)
      $splitters = false;
    $this->setSplitters($splitters);
  }

  /**
   * Setzt den Layout-Modus (headline oder sidebar).
   *
   * <dl>
   * <dt><tt>GhGuiBorderContainer::LAYOUT_HEADLINE</tt></dt>
   * <dd>Der obere und der untere Bereich nehmen die gesamte Breite ein und
   * liegen über bzw. unter dem linken und rechten Bereich, sowie dem
   * Inhaltsbereich.</dd>
   * <dt><tt>GhGuiBorderContainer::LAYOUT_SIDEBAR</tt></dt>
   * <dd>Der linke und der rechte Bereich nehmen die gesamte Höhe ein. Der obere
   * und untere Bereich liegen zwischen dem linken und rechten Bereich.</dd>
   * </dl>
   *
   * @param string $mode Layout-Modus
   * @return GhGuiBorderContainer liefert den GhGuiBorderContainer zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setLayoutMode($mode)
  {
    $this->setWidgetProperty('design', $mode);

    return $this;
  }

  /**
   * Liefert den Layout-Modus des Containers.
   *
   * @return string
   */
  public function getLayoutMode()
  {
    return $this->getWidgetProperty('design');
  }

  /**
   * Legt fest, ob Trennlinien angezeigt werden sollen, mit denen (durch
   * Ziehen)die Größe der Bereiche verändert werden kann.
   *
   * @param boolean $splitters true: Trennlinien für Größenänderung anzeigen,
   *   false: keine Größenänderung ermöglichen
   * @return GhGuiBorderContainer liefert den GhGuiBorderContainer zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setSplitters($splitters)
  {
    if($splitters)
      $this->setWidgetProperty('liveSplitters', false);
    elseif(isset($this->_widgetProperties['liveSplitters']))
      unset($this->_widgetProperties['liveSplitters']);

    return $this;
  }

  /**
   * Liefert die Einstellung, ob Trennlinien zur Größenänderung der Bereiche
   * angezeigt werden.
   *
   * @return boolean
   */
  public function getSplitters()
  {
    return $this->getWidgetProperty('liveSplitters') === false;
  }

  /**
   * Fügt eine Komponente zum oberen Bereich hinzu.
   * Optional kann die Höhe des oberen Bereichs angegben werden. Ein
   * numerischer Wert gibt die Höhe in Pixeln an, ein Wert mit Prozentzeichen
   * am Ende gibt eine prozentuale Höhe an.
   *
   * @param GhGuiComponent $component die hinzuzufügende Komponente
   * @param string         $height    die Höhe des oberen Bereichs
   * @return GhGuiComponent die hinzugefügte Komponente
   */
  public function addTopComponent(GhGuiComponent $component, $height = null)
  {
    $component->setWidgetProperty('region', 'top');
    $component->_setParentContainer($this);

    if($height)
      $component->setStyle('height', $height);

    $this->_top[] = $component;

    return $component;
  }

  /**
   * Fügt eine Komponente zum rechten Bereich hinzu.
   * Optional kann die Breite des rechten Bereichs angegben werden. Ein
   * numerischer Wert gibt die Breite in Pixeln an, ein Wert mit Prozentzeichen
   * am Ende gibt eine prozentuale Breite an.
   *
   * @param GhGuiComponent $component die hinzuzufügende Komponente
   * @param string         $width     die Breite des rechten Bereichs
   * @return GhGuiComponent die hinzugefügte Komponente
   */
  public function addRightComponent(GhGuiComponent $component, $width = null)
  {
    $component->setWidgetProperty('region', 'right');
    $component->_setParentContainer($this);

    if($width)
      $component->setStyle('width', $width);

    $this->_right[] = $component;

    return $component;
  }

  /**
   * Fügt eine Komponente zum unteren Bereich hinzu.
   * Optional kann die Höhe des unteren Bereichs angegben werden. Ein
   * numerischer Wert gibt die Höhe in Pixeln an, ein Wert mit Prozentzeichen
   * am Ende gibt eine prozentuale Höhe an.
   *
   * @param GhGuiComponent $component die hinzuzufügende Komponente
   * @param string         $height    die Höhe des unteren Bereichs
   * @return GhGuiComponent die hinzugefügte Komponente
   */
  public function addBottomComponent(GhGuiComponent $component, $height = null)
  {
    $component->setWidgetProperty('region', 'bottom');
    $component->_setParentContainer($this);

    if($height)
      $component->setStyle('height', $height);

    $this->_bottom[] = $component;

    return $component;
  }

  /**
   * Fügt eine Komponente zum linken Bereich hinzu.
   * Optional kann die Breite des linken Bereichs angegben werden. Ein
   * numerischer Wert gibt die Breite in Pixeln an, ein Wert mit Prozentzeichen
   * am Ende gibt eine prozentuale Breite an.
   *
   * @param GhGuiComponent $component die hinzuzufügende Komponente
   * @param string         $width     die Breite des linken Bereichs
   * @return GhGuiComponent die hinzugefügte Komponente
   */
  public function addLeftComponent(GhGuiComponent $component, $width = null)
  {
    $component->setWidgetProperty('region', 'left');
    $component->_setParentContainer($this);

    if($width)
      $component->setStyle('width', $width);

    $this->_left[] = $component;

    return $component;
  }

  /**
   * Fügt eine Komponente zum mittleren Bereich hinzu.
   * Optional kann die Breite und Höhe des mittleren Bereichs angegben werden.
   * Ein numerischer Wert gibt die Breite bzw. Höhe in Pixeln an, ein Wert mit
   * Prozentzeichen am Ende gibt eine prozentuale Breite bzw. Höhe an.
   *
   * @param GhGuiComponent $component die hinzuzufügende Komponente
   * @param string         $width     die Breite des mittleren Bereichs
   * @param string         $height    die Höhe des mittleren Bereichs
   * @return GhGuiComponent die hinzugefügte Komponente
   */
  public function addCenterComponent(GhGuiComponent $component, $width = null, $height = null)
  {
    $component->setWidgetProperty('region', 'center');

    if($width)
      $component->setStyle('width', $width);
    if($height)
      $component->setStyle('height', $height);

    $component->_setParentContainer($this);
    $this->_center[] = $component;

    return $component;
  }

  /**
   * Liefert die Komponenten, die zum oberen Bereich hinzugefügt wurden.
   *
   * @return array
   */
  public function getTopComponents()
  {
    return $this->_top;
  }

  /**
   * Liefert die Komponenten, die zum rechten Bereich hinzugefügt wurden.
   *
   * @return array
   */
  public function getRightComponents()
  {
    return $this->_right;
  }

  /**
   * Liefert die Komponenten, die zum unteren Bereich hinzugefügt wurden.
   *
   * @return array
   */
  public function getBottomComponents()
  {
    return $this->_bottom;
  }

  /**
   * Liefert die Komponenten, die zum linken Bereich hinzugefügt wurden.
   *
   * @return array
   */
  public function getLeftComponents()
  {
    return $this->_left;
  }

  /**
   * Liefert die Komponenten, die zum mittleren Bereich hinzugefügt wurden.
   *
   * @return array
   */
  public function getCenterComponents()
  {
    return $this->_center;
  }

  /**
   * Diese Methode lieferte bislang den Container, der die Komponenten für den
   * oberen Bereich des Border-Layouts enthielt.
   *
   * @deprecated Die Komponenten werden in der aktuellen Version ohne
   * zusätzlichen Container zum GhGuiBorderContainer hinzugefügt.
   * Die Änderungen, die bisher am Container vorgenommen wurden, müssen nun
   * direkt an den hinzugefügten Komponenten vorgenommen werden.
   * [paragraph]
   * Diese Methode liefert in der aktuellen Version ein leeres GhGuiPanel
   * zurück um Fehler bei der Ausführung zu vermeiden. Änderungen an der
   * zurückgelieferten Komponente haben jedoch keine Auswirkungen.
   *
   * @return GhGuiPanel
   */
  public function getTopContainer()
  {
    return new GhGuiPanel();
  }

  /**
   * Diese Methode lieferte bislang den Container, der die Komponenten für den
   * rechten Bereich des Border-Layouts enthielt.
   *
   * @deprecated Die Komponenten werden in der aktuellen Version ohne
   * zusätzlichen Container zum GhGuiBorderContainer hinzugefügt.
   * Die Änderungen, die bisher am Container vorgenommen wurden, müssen nun
   * direkt an den hinzugefügten Komponenten vorgenommen werden.
   * [paragraph]
   * Diese Methode liefert in der aktuellen Version ein leeres GhGuiPanel
   * zurück um Fehler bei der Ausführung zu vermeiden. Änderungen an der
   * zurückgelieferten Komponente haben jedoch keine Auswirkungen.
   *
   * @return GhGuiPanel
   */
  public function getRightContainer()
  {
    return new GhGuiPanel();
  }

  /**
   * Diese Methode lieferte bislang den Container, der die Komponenten für den
   * unteren Bereich des Border-Layouts enthielt.
   *
   * @deprecated Die Komponenten werden in der aktuellen Version ohne
   * zusätzlichen Container zum GhGuiBorderContainer hinzugefügt.
   * Die Änderungen, die bisher am Container vorgenommen wurden, müssen nun
   * direkt an den hinzugefügten Komponenten vorgenommen werden.
   * [paragraph]
   * Diese Methode liefert in der aktuellen Version ein leeres GhGuiPanel
   * zurück um Fehler bei der Ausführung zu vermeiden. Änderungen an der
   * zurückgelieferten Komponente haben jedoch keine Auswirkungen.
   *
   * @return GhGuiPanel
   */
  public function getBottomContainer()
  {
    return new GhGuiPanel();
  }

  /**
   * Diese Methode lieferte bislang den Container, der die Komponenten für den
   * linken Bereich des Border-Layouts enthielt.
   *
   * @deprecated Die Komponenten werden in der aktuellen Version ohne
   * zusätzlichen Container zum GhGuiBorderContainer hinzugefügt.
   * Die Änderungen, die bisher am Container vorgenommen wurden, müssen nun
   * direkt an den hinzugefügten Komponenten vorgenommen werden.
   * [paragraph]
   * Diese Methode liefert in der aktuellen Version ein leeres GhGuiPanel
   * zurück um Fehler bei der Ausführung zu vermeiden. Änderungen an der
   * zurückgelieferten Komponente haben jedoch keine Auswirkungen.
   *
   * @return GhGuiPanel
   */
  public function getLeftContainer()
  {
    return new GhGuiPanel();
  }

  /**
   * Diese Methode lieferte bislang den Container, der die Komponenten für den
   * mittleren Bereich des Border-Layouts enthielt.
   *
   * @deprecated Die Komponenten werden in der aktuellen Version ohne
   * zusätzlichen Container zum GhGuiBorderContainer hinzugefügt.
   * Die Änderungen, die bisher am Container vorgenommen wurden, müssen nun
   * direkt an den hinzugefügten Komponenten vorgenommen werden.
   * [paragraph]
   * Diese Methode liefert in der aktuellen Version ein leeres GhGuiPanel
   * zurück um Fehler bei der Ausführung zu vermeiden. Änderungen an der
   * zurückgelieferten Komponente haben jedoch keine Auswirkungen.
   *
   * @return GhGuiPanel
   */
  public function getCenterContainer()
  {
    return new GhGuiPanel();
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  public function _getAllComponents()
  {
    return array_merge($this->_top, $this->_left, $this->_center, $this->_right, $this->_bottom);
  }

  /**
   * Initialisiert die Komponente und alle in ihr enthaltenen Komponenten für
   * das Abrufen des HTML-Codes.
   *
   * Falls die Komponente auf andere Komponenten zugreifen muss um ihre HTML
   * Ausgabe zu bestimmen, so kann diese Methode entsprechend überladen werden.
   *
   * Liefert true, falls die Komponente initialisiert wurde, false, falls die
   * Komponente bereits initialisiert war.
   *
   * @return boolean
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    if(empty($this->_center))
    {
      $message = 'The center area of GhGuiBorderContainer must not be empty';

      if($this->_creationTrace)
        $message .= ' (created in ' . $this->_creationTrace['file'] . ':' . $this->_creationTrace['line'] . ')';

      throw new GhGuiException($message);
    }

    // Splitter anzeigen:
    if($this->getSplitters())
    {
      foreach($this->_getAllComponents() as $component)
        $component->setWidgetProperty('splitter', true);
    }

    // Prozentuale Größen auf Gesamtsumme 100% begrenzen:
    // Höhe:
    $total = 0;
    $components = array();
    foreach(array_merge($this->_top, $this->_center, $this->_bottom) as $component)
    {
      $percent = $this->_getPercentValue($component->getStyle('height'));
      if($percent !== null && $percent > 0)
      {
        $total += $percent;
        $components[] = $component;
      }
    }
    if($total > 100)
    {
      $factor = 100 / $total;
      foreach($components as $component)
        $component->setStyle('height', round($this->_getPercentValue($component->getStyle('height')) * $factor, 2) . '%');
    }
    // Breite:
    $total = 0;
    $components = array();
    foreach(array_merge($this->_left, $this->_center, $this->_right) as $component)
    {
      $percent = $this->_getPercentValue($component->getStyle('width'));
      if($percent !== null && $percent > 0)
      {
        $total += $percent;
        $components[] = $component;
      }
    }
    if($total > 100)
    {
      $factor = 100 / $total;
      foreach($components as $component)
        $component->setStyle('width', round($this->_getPercentValue($component->getStyle('width')) * $factor, 2) . '%');
    }

    return true;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'gutters':
      case 'liveSplitters':
        return 'boolean';
    }

    return parent::__getWidgetPropertyType($property);
  }
}