<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Container, der Komponenten spaltenweise darstellt.
 *
 * Dieser Container arrangiert die hinzugefügten Komponenten in einer
 * festgelegten Anzahl Spalten. Optional kann jede Komponente mit einer
 * Beschriftung hinzugefügt werden, die in einer zusätzlichen Spalte vor dem
 * eigentlichen Inhalt angezeigt werden kann.
 *
 * @package GhGui
 * @subpackage container
 * @category PHP-GUI-Strukturkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiTableContainer extends GhGuiContainer
{
  /**
   * Vertikale Tabellenausrichtung.
   * @var const
   */
  const ORIENTATION_VERTICAL = 'vert';

  /**
   * Horizontale Tabellenausrichtung.
   * @var const
   */
  const ORIENTATION_HORIZONTAL = 'horiz';

  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('GhGuiTableContainer');

  /**
   * Widget Klassenname dieser Komponente.
   * @var string
   */
  protected $_widgetClass = 'dijit.layout.ContentPane';

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dojox.layout.TableContainer');

  /**
   * Das interne Widget, das die Tabelle repräsentiert.
   * @var GhGuiPanel
   */
  protected $_table;

  /**
   * Konstruktor.
   *
   * Optional kann die Anzahl der Spalten für die Tabelle gesetzt werden,
   * sowie die Anzeige der Beschriftungen aktiviert und die Ausrichtung
   * festgelegt werden.
   *
   * Wird Anzeige von Beschriftungen aktiviert, so werden die Beschriftungen
   * der hinzugefügten Komponenten in einer zusätzlichen Tabellenspalte (pro
   * normaler Spalte) ausgegeben. Die Anzahl der tatsächlich dargestellten
   * Spalten ist dann doppelt so hoch wie festgelegt.
   *
   * Bei horizontaler Ausrichtung werden Elemente spaltenweise hinzugefügt und
   * Beschriftungen werden links von den Inhalten dargestellt. Bei vertikaler
   * Ausrichtung werden Elemente zeilenweise hinzugefügt und Beschriftungen
   * werden über den Inhalten dargestellt.
   * Die Vorgabe für die Tabellenausrihtung ist die horizontale Ausrichtung.
   *
   * <dl>
   * <dt><tt>GhGuiTableContainer::ORIENTATION_HORIZONTAL</tt></dt>
   * <dd>Horizontale Ausrichtung der Tabelle.</dd>
   * <dt><tt>GhGuiTableContainer::ORIENTATION_VERTICAL</tt></dt>
   * <dd>Vertikale Ausrichtung der Tabelle.</dd>
   * </dl>
   *
   * @param int      $columns     Spaltenanzahl
   * @param boolean $showCaptions true: Beschriftungen anzeigen,
   *   false: Beschriftungen nicht anzeigen
   * @param string $orientation Ausrichtung
   */
  public function __construct($columns = null, $showCaptions = null, $orientation = null)
  {
    parent::__construct();

    $this->_table = new GhGuiPanel();
    $this->_table->_setWidgetClass('dojox.layout.TableContainer');

    if($columns === null)
      $columns = 1;

    $this->setColumns($columns);

    if($showCaptions === null)
      $showCaptions = false;

    $this->setShowCaptions($showCaptions);

    if($orientation === null)
      $orientation = self::ORIENTATION_HORIZONTAL;
    
    $this->setOrientation($orientation);
  }

  /**
   * Setzt die Anzahl der Spalten der Tabelle.
   *
   * Falls die Option, Beschriftungen anzuzeigen gesetzt ist (siehe
   * setShowCaptions()), so werden doppelt so viele Spalten dargestellt wie
   * festgelegt (da die Beschriftungen in jeweils einer zusätzlichen Spalte
   * angezeigt werden).
   *
   * @param int $columns Spaltenanzahl
   * @return GhGuiTableContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setColumns($columns)
  {
    $this->_table->setWidgetProperty('cols', (int)$columns);

    return $this;
  }

  /**
   * Liefert die Anzahl der Spalten der Tabelle.
   *
   * Falls die Option, Beschriftungen anzuzeigen gesetzt ist (siehe
   * setShowCaptions()), so ist die tatsächlich dargestellte Anzahl Spalten
   * entsprechend doppelt so hoch wie der zurück gelieferte Wert.
   *
   * @return int
   */
  public function getColumns()
  {
    return $this->_table->getWidgetProperty('cols');
  }

  /**
   * Legt fest, ob Beschriftungen angezeigt werden sollen.
   *
   * Wird diese Einstellung auf true gesetzt, so werden die Beschriftungen der
   * hinzugefügten Komponenten in einer zusätzlichen Tabellenspalte (pro
   * normaler Spalte) ausgegeben.
   *
   * @param boolean $showCaptions true: Beschriftungen anzeigen,
   *   false: Beschriftungen nicht anzeigen
   * @return GhGuiTableContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setShowCaptions($showCaptions)
  {
    $this->_table->setWidgetProperty('showLabels', $showCaptions ? true : false);

    return $this;
  }

  /**
   * Liefert die Einstellung, ob Beschriftungen angezeigt werden sollen.
   *
   * @return boolean
   */
  public function getShowCaptions()
  {
    return $this->_table->getWidgetProperty('showLabels');
  }

  /**
   * Setzt die Ausrichtung der Tabelle (horizontal oder vertikal).
   *
   * Bei horizontaler Ausrichtung werden Elemente spaltenweise hinzugefügt und
   * Beschriftungen werden links von den Inhalten dargestellt. Bei vertikaler
   * Ausrichtung werden Elemente zeilenweise hinzugefügt und Beschriftungen
   * werden über den Inhalten dargestellt.
   *
   * <dl>
   * <dt><tt>GhGuiTableContainer::ORIENTATION_HORIZONTAL</tt></dt>
   * <dd>Horizontale Ausrichtung der Tabelle.</dd>
   * <dt><tt>GhGuiTableContainer::ORIENTATION_VERTICAL</tt></dt>
   * <dd>Vertikale Ausrichtung der Tabelle.</dd>
   * </dl>
   *
   * @param string $orientation Ausrichtung
   * @return GhGuiTableContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setOrientation($orientation)
  {
    $this->_table->setWidgetProperty('orientation', $orientation);

    return $this;
  }

  /**
   * Liefert die Ausrichtung der Tabelle.
   *
   * @return string
   */
  public function getOrientation()
  {
    return $this->_table->getWidgetProperty('orientation');
  }

  /**
   * Setzt den Abstand der Tabellenzellen.
   *
   * @param int $spacing Zellenabstand in Pixeln
   * @return GhGuiTableContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCellSpacing($spacing)
  {
    $this->_table->setWidgetProperty('spacing', (int)$spacing);

    return $this;
  }

  /**
   * Liefert den Zellenabstand der Tabelle.
   *
   * @return int
   */
  public function getCellSpacing()
  {
    return $this->_table->getWidgetProperties('spacing');
  }

  /**
   * Setzt die Breite der Beschriftungen der Tabelle.
   *
   * @param int|string $width Breite in Pixeln oder als Prozentangabe
   * @return GhGuiTableContainer liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setLabelWidth($width)
  {
    $this->_table->setWidgetProperty('labelWidth', $width);

    return $this;
  }

  /**
   * Liefert die Breite der Beschriftungen der Tabelle.
   *
   * @return int|string
   */
  public function getLabelWidth()
  {
    return $this->_table->getWidgetProperty('labelWidth');
  }

  /**
   * Fügt eine Komponente hinzu.
   *
   * Optional kann eine Beschriftung angegeben werden. Diese wird als
   * zusätzliche Tabellenspalte angezeigt wenn setShowCaptions() gesetzt
   * wurde, andernfalls hat die Beschriftung keine Auswirkung.
   *
   * @param GhGuiComponent $component   Komponente
   * @param string         $caption     Beschriftung
   * @param boolean        $fillWidth   Inhalt auf Breite der Tabellenzelle ausdehnen
   * @param boolean        $fillHeight  Inhalt auf Höhe der Tabellenzelle ausdehnen
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addCell(GhGuiComponent $component, $caption = null, $fillWidth = null, $fillHeight = null)
  {
    if($fillWidth !== null)
      $component->setFillWidth($fillWidth);

    if($fillHeight !== null)
      $component->setFillHeight($fillHeight);

    if($caption !== null)
    {
      if($component->getWidgetProperty('label') !== null)
      {
        $wrapper = new GhGuiPanel();
        $wrapper->setWidgetProperty('label', $caption);
        $wrapper->addComponent($component);

        if($fillWidth !== null)
          $wrapper->setFillWidth($fillWidth);

        if($fillHeight !== null)
          $wrapper->setFillHeight($fillHeight);

        $this->_table->addComponent($wrapper);
      }
      else
      {
        $component->setWidgetProperty('label', $caption);
        $this->_table->addComponent($component);
      }
    }
    else
      $this->_table->addComponent($component);

    return $component;
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    return array($this->_table);
  }
}