<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Spalte für GhGuiDataGrid.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiDataGridColumn
{
  /**
   * Konstante für unbestimmte Variablentypen.
   */
  const TYPE_UNDEFINED = null;

  /**
   * Konstante für String-Typen (Zeichenketten).
   */
  const TYPE_STRING = 'string';

  /**
   * Konstante für Integer-Typen (Ganzzahlen).
   */
  const TYPE_INT = 'int';

  /**
   * Konstante für Float-Typen (Dezimalzahlen).
   */
  const TYPE_FLOAT = 'float';

  /**
   * Konstante für Datums-Typen.
   */
  const TYPE_DATE = 'date';

  /**
   * ID der Spalte.
   * @var string
   */
  protected $_id;

  /**
   * Objekteigenschaft der Datenobjekte für den Wert dieser Spalte.
   * @var string
   */
  protected $_property;

  /**
   * Variablentyp dieser Spalte.
   * @var string
   */
  protected $_type = self::TYPE_UNDEFINED;

  /**
   * Spaltentitel.
   * @var string
   */
  protected $_caption;

  /**
   * Breite der Spalte (Pixel, oder Prozentangabe).
   * @var string
   */
  protected $_width;

  /**
   * CSS Angaben für den Spaltenkopf.
   * @var string
   */
  protected $_headerStyle;

  /**
   * CSS Angaben für die Tabellenzellen dieser Spalte.
   * @var string
   */
  protected $_cellStyle;

  /**
   * Formatierer für die Zellen dieser Spalte.
   * Dies ist ein String mit einer Javascript-Funktion, in der Regel eine der
   * Konstanten der GhGuiFormatter Klasse.
   * @var string
   */
  protected $_cellFormatter;

  /**
   * Text, der den Daten in den Tabellenzellen vorangestellt wird.
   * @var string
   */
  protected $_dataPrefix;

  /**
   * Text, der an die Daten in den Tabellenzellen angehängt wird.
   * @var string
   */
  protected $_dataSuffix;

  /**
   * Konstruktor.
   *
   * Spalten ohne Beschriftung (Spaltentitel) werden nicht im Data-Grid
   * angezeigt. Solche Spalten hinzuzufügen macht Sinn, wenn man den Wert der
   * Spalte als Wert für das Formularfeld nutzen möchte (siehe
   * setValueProperty()).
   *
   * Zusätzlich zu der Objekteigenschaft für diese Spalte, der Beschriftung
   * und der Spaltenbreite kann ein Formatierer für die Zellen dieser Spalte
   * übergeben werden.
   * Der Formatierer ist eine Javascript Funktion, die als Parameter die
   * Daten der Tabellenzelle erhält und einen String mit dem darzustellenden
   * Inhalt zurückliefert.
   *
   * Als vordefinierte Formatierer können die Konstanten der Klasse
   * [classes]GhGuiFormatter[/classes] verwendet werden.
   *
   * @param string         $property      Objekteigenschaft für die Werte dieser Spalte
   * @param string         $caption       Beschriftung (Spaltentitel)
   * @param string         $width         Spaltenbreite (Pixel oder Prozentangabe)
   * @param GhGuiFormatter $cellFormatter Zellenformatierer (Javascript-Funktion)
   */
  public function __construct($property, $caption = null, $width = null, GhGuiFormatter $cellFormatter = null)
  {
    $this->setProperty($property);

    if($caption !== null)
      $this->setCaption($caption);

    if($width !== null)
      $this->setWidth($width);

    if($cellFormatter !== null)
      $this->setCellFormatter($cellFormatter);
  }

  /**
   * Setzt die ID für diese Spalte.
   *
   * @param string $id ID (wird als DOM Knoten ID verwendet)
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setId($id)
  {
    $this->_id = $id;

    return $this;
  }

  /**
   * Liefert die ID für diese Spalte.
   *
   * @return string
   */
  public function getId()
  {
    if(!$this->_id)
      $this->_id = GhGui::createId();

    return $this->_id;
  }

  /**
   * Setzt die Objekteigenschaft der Datenobjekte, die für den Inhalt der
   * Tabellenzellen dieser Spalte verwendet wird.
   *
   * @param string $property Objekteigenschaft
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setProperty($property)
  {
    $this->_property = $property;

    return $this;
  }

  /**
   * Liefert die Objekteigenschaft der Datenobjekte, die für den Inhalt der
   * Tabellenzellen dieser Spalte verwendet wird.
   *
   * @return string
   */
  public function getProperty()
  {
    return $this->_property;
  }

  /**
   * Setzt den Variablentyp dieser Spalte.
   * Als Wert kann eine der GhGuiDataGridColumn::TYPE_* Konstanten übergeben werden.
   *
   * @param string $type Variablentyp
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setType($type)
  {
    $this->_type = $type;

    return $this;
  }

  /**
   * Liefert den Variablentyp dieser Spalte.
   * Falls kein Typ festgelegt wurde, so wird null geliefert.
   *
   * @return string
   */
  public function getType()
  {
    return $this->_type;
  }

  /**
   * Setzt den Spaltentitel.
   *
   * @param string $caption Spaltentitel
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCaption($caption)
  {
    $this->_caption = $caption;

    return $this;
  }

  /**
   * Liefert den Spaltentitel.
   *
   * @return string
   */
  public function getCaption()
  {
    return $this->_caption;
  }

  /**
   * Setzt die Breite dieser Spalte.
   * Die Angabe kann numerisch sein (Breite in Pixeln) oder mit einem
   * Prozentzeichen enden (Prozentangabe).
   *
   * @param string $width Spaltenbreite
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setWidth($width)
  {
    $this->_width = $width;

    return $this;
  }

  /**
   * Liefert die Spaltenbreite.
   *
   * @return string
   */
  public function getWidth()
  {
    return $this->_width;
  }

  /**
   * Liefert die Sichtbarkeit der Spalte zurück.
   *
   * Das Ergebnis true bedeutet, dass die Spalte sichtbar ist und im Data-Grid
   * angezeigt wird, false bedeutet, dass die Spalte unsichtbar ist und nicht
   * angezeigt wird.
   *
   * @return boolean
   */
  public function getVisible()
  {
    $caption = $this->getCaption();

    return strlen($caption) > 0;
  }

  /**
   * Setzt CSS Angaben für das Erscheinungsbild des Spaltenkopfes (z.B.
   * Textausrichtung).
   *
   * @param string $css CSS Angaben
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setStyle($css)
  {
    $this->_headerStyle = $css;

    return $this;
  }

  /**
   * Liefert die CSS Angaben für den Spaltenkopf.
   *
   * @return string
   */
  public function getStyle()
  {
    return $this->_headerStyle;
  }

  /**
   * Setzt CSS Angaben für das Erscheinungsbild der Tabellenzellen dieser
   * Spalte (z.B. Farbe, Textausrichtung).
   *
   * @param string $css CSS Angaben
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCellStyle($css)
  {
    $this->_cellStyle = $css;

    return $this;
  }

  /**
   * Liefert die CSS Angaben für die Tabellenzellen dieser Spalte.
   *
   * @return string
   */
  public function getCellStyle()
  {
    return $this->_cellStyle;
  }

  /**
   * Setzt den Formatierer für die Tabellenzellen dieser Spalte.
   * Der Formatierer ist eine Javascript Funktion, die als Parameter die
   * Daten der Tabellenzelle erhält und einen String mit dem darzustellenden
   * Inhalt zurückliefert.
   *
   * Als vordefinierte Formatierer können die Konstanten der Klasse
   * [classes]GhGuiFormatter[/classes] verwendet werden.
   *
   * @param GhGuiFormatter $formatter Zellenformatierer
   * @return GhGuiDataGridColumn liefert die Spalte (DataGridColumn) zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setCellFormatter(GhGuiFormatter $formatter)
  {
    $this->_cellFormatter = $formatter;

    return $this;
  }

  /**
   * Liefert den Zellenformatierer für die Tabellenzellen dieser Spalte.
   *
   * @return GhGuiFormatter
   */
  public function getCellFormatter()
  {
    return $this->_cellFormatter;
  }
}
