<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Basisklasse für Formular-Komponenten.
 *
 * Diese Klasse dient als Basis für Komponenten, die innerhalb eines
 * Formulars (einer HTML Form) verwendet werden, um einen Wert (durch
 * Auswahl oder Eingabe) beim Absenden des Formulars zu übermitteln.
 *
 * Abgeleitete Klassen müssen die Methode _getAutoSubmitEvents() überladen,
 * damit die Komponente die auto-submit Funktion unterstützt.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
abstract class GhGuiFormComponent extends GhGuiComponent
{
  /**
   * Vorgabewert des Formularfeldes.
   * Dieser Wert wird verwendet, wenn für das Formularfeld noch kein Wert
   * (z.B. aus einem vorhergehenden GET oder POST) vorliegt.
   * @var string
   */
  protected $_defaultValue;

  /**
   * Einstellung, ob der aktuelle POST/GET Wert für den Wert des Formularfelds
   * verwendet werden soll.
   */
  protected $_useCurrentValue = true;

  /**
   * Events, die ein automatisches Absenden des Formulars auslösen.
   * @var array
   */
  protected $_autoSubmitEvents = array();

  /**
   * Auto-submit Event-Handler.
   * Wenn diese Variable leer ist, so ist Auto-submit für diese Komponente
   * deaktiviert, andernfalls ist es aktiviert.
   * @var GhGuiEventAutoSubmit
   */
  protected $_autoSubmitEventHandler;

  /**
   * Legt fest, ob die Attribute 'name' und 'value' für den HTML-Knoten erzeugt
   * werden.
   * @var boolean
   */
  protected $_createNameAndValueAttributes = true;

  /**
   * Legt fest, ob die Eigenschaften 'name' und 'value' für das Javascript
   * Widget erzeugt werden.
   * @var boolean
   */
  protected $_createNameAndValueWidgetProperties = true;


  /**
   * Konstruktor.
   *
   * Optional kann der Name des Eingabefeldes festgelegt werden (siehe
   * setName()).
   * Zusätzlich können der Wert (siehe setValue()) und der Vorgabewert (siehe
   * setDefaultValue()) für das Formularfeld angegeben werden und die
   * auto-submit Funktion aktiviert werden (siehe setAutoSubmit()).
   *
   * @param string  $name         Name des Formularfeldes
   * @param string  $value        Wert für das Formularfeldes
   * @param string  $defaultValue Vorgabewert für das Formularfeld
   * @param boolean $autoSubmit   true: auto-submit, false: kein auto-submit
   */
  public function __construct($name = null, $value = null, $defaultValue = null, $autoSubmit = null)
  {
    parent::__construct();

    if($name !== null)
      $this->setName($name);
    if($value !== null)
      $this->setValue($value);
    if($defaultValue !== null)
      $this->setDefaultValue($defaultValue);
    if($autoSubmit !== null)
      $this->setAutoSubmit($autoSubmit);
  }

  /**
   * Liefert die ID für das Formularelement, das den Wert dieser Komponente
   * enthält.
   * Bei sichtbaren Eingabeelementen (z.B. Textfeldern) ist dies in der Regel
   * die ID des HTML Haupt-Knotens der Komponente. Bei anderen Komponenten ist
   * dies die ID eines versteckten Formularfeldes (hidden field), in das die
   * Komponente bei Auswahl oder Eingabe eines Wertes, den Wert speichert.
   *
   * @return string dom node id of the component's value node
   */
  public function getValueNodeId()
  {
    return $this->getId();
  }

  /**
   * Liefert den Namen des Formularfeldes.
   *
   * @return string name
   */
  public function getName()
  {
    return $this->getWidgetProperty('name');
  }

  /**
   * Setzt den Namen des Eingabefeldes.
   * Der Name wird für das 'name' Attribut des HTML Formularelements
   * verwendet. Unter diesem Namen werden die Daten im Formular abgeschickt.
   *
   * Der Name kann in Array-Schreibweise (mit eckigen Klammern) angegeben
   * werden, um z.B. zusammengehörige Daten zu bündeln.
   *
   * @param string $name Name des Formularfeldes
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setName($name)
  {
    $this->setWidgetProperty('name', $name);

    return $this;
  }

  /**
   * Legt fest, ob das Formular beim Ändern des Wertes automatisch abgeschickt
   * wird.
   *
   * Falls dieser Wert auf true eingestellt wird, dann wird das Formular, in
   * dem sich diese Komponente befindet, beim Ändern des Wertes bzw. beim
   * Auswählen eines Wertes automatisch abgeschickt.
   *
   * @param boolean $autoSubmit true: Formular automatisch abschicken,
   *   false: Formular nicht automatisch abschicken
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setAutoSubmit($autoSubmit)
  {
    if(!$autoSubmit)
      $this->_autoSubmitEventHandler = null;
    elseif($this->_autoSubmitEventHandler === null)
      $this->_autoSubmitEventHandler = new GhGuiEventAutoSubmit();

    return $this;
  }

  /**
   * Liefert die auto-submit Einstellung.
   *
   * @return boolean
   */
  public function getAutoSubmit()
  {
    return $this->_autoSubmitEventHandler !== null;
  }

  /**
   * Liefert die Events der Komponente, die ein auto-submit auslösen.
   *
   * @return array
   */
  public function getAutoSubmitEvents()
  {
    return $this->_autoSubmitEvents;
  }

  /**
   * Liefert alle Event-Handler dieser Komponente.
   *
   * Das Ergebnis ist ein Array, dessen Schlüssel Event-Namen und dessen Werte
   * Arrays mit Event-Handlern (Javascript-Funktionen) sind.
   *
   * @param string $event Name des Events (falls null werden alle Event-Handler zurückgeliefert)
   * @return array
   */
  public function getEventHandlers($event = null)
  {
    // Die Auto-submit Event-Handler immer am Ende der Event-Handler anhängen:
    $handlers = parent::getEventHandlers($event);

    // Kein Auto-submit:
    if(!$this->getAutoSubmit())
      return $handlers;

    // Nur Event-Handler für bestimmtes Event:
    if($event !== null)
    {
      if(in_array($event, $this->_autoSubmitEvents))
        $handlers[] = $this->_autoSubmitEventHandler;
      return $handlers;
    }

    // Alle Event-Handler:
    foreach($this->_autoSubmitEvents as $event)
    {
      if(!isset($handlers[$event]))
        $handlers[$event] = array($this->_autoSubmitEventHandler);
      else
        $handlers[$event][] = $this->_autoSubmitEventHandler;
    }

    return $handlers;
  }

  /**
   * Liefert alle Attribute für den Haupt-DOM-Knoten dieser Komponente.
   *
   * Form-Komponenten liefern zusätzlich (sofern sie gesetzt sind) den
   * Namen des Formularfelds ('name') und den aktuellen Wert ('value').
   *
   * @return array
   */
  protected function _getAllAttributes()
  {
    $attributes = parent::_getAllAttributes();

    $name = $this->getName();
    if($name !== null && $this->_createNameAndValueAttributes)
      $attributes['name'] = $name;

    $value = $this->getCurrentValue();
    if($value !== null && $this->_createNameAndValueAttributes)
      $attributes['value'] = $value;

    return $attributes;
  }

  /**
   * Setzt den Wert dieses Formularfeldes.
   *
   * @param string $value Wert für das Formularfeld
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setValue($value)
  {
    $this->setWidgetProperty('value', $value);

    return $this;
  }

  /**
   * Liefert den Wert des Formularfeldes.
   *
   * @return string
   */
  public function getValue()
  {
    return $this->getWidgetProperty('value');
  }

  /**
   * Setzt den Vorgabewert dieses Formularfeldes.
   * Dieser Wert wird verwendet, wenn für das Formularfeld noch kein Wert
   * (z.B. aus einem vorhergehenden GET oder POST) vorliegt.
   *
   * @param string $value Vorgabewert für das Formularfeld
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setDefaultValue($value)
  {
    $this->_defaultValue = $value;

    return $this;
  }

  /**
   * Liefert den Vorgabewert dieses Formularfeldes.
   *
   * @return string
   */
  public function getDefaultValue()
  {
    return $this->_defaultValue;
  }

  /**
   * Legt fest, ob der aktuelle POST/GET Wert für den Wert des Formularfelds
   * verwendet werden soll.
   *
   * Ist diese Einstellung aktiv, so wird der Wert, der in den POST bzw. GET
   * für den Namen des Formularfelds vorliegt, automatisch für dessen Wert
   * genutzt. D.h. die Komponente wird mit dem zuletzt abgesendeten Wert wieder
   * initialisiert.
   *
   * @param boolean $useCurrentValue true: aktuellen Wert verwenden,
   *   false: aktuellen Wert nicht verwenden
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setUseCurrentValue($useCurrentValue)
  {
    $this->_useCurrentValue = $useCurrentValue;

    return $this;
  }

  /**
   * Liefert die Einstellung, ob der aktuelle POST/GET Wert für den Wert des
   * Formularfelds verwendet werden soll.
   *
   * @return boolean
   */
  public function getUseCurrentValue()
  {
    return $this->_useCurrentValue;
  }

  /**
   * Liefert den aktuellen Wert dieses Formularfeldes.
   *
   * Falls ein Wert explizit gesetzt wurde (setValue()), so wird dieser
   * zurück geliefert. Andernfalls wird der Formularwert aus einem
   * vorhergehenden GET oder POST geliefert, falls er existiert. Ansonsten
   * wird der Vorgabewert (setDefaultValue()) geliefert.
   *
   * @return string
   */
  public function getCurrentValue()
  {
    $value = $this->getValue();
    if ($value !== null)
      return $value;
    elseif($this->getUseCurrentValue() && $this->getName())
    {
      $value = GhCoreGlobals::getValue($this->getName());

      if($value !== null)
        return $value;
    }

    return $this->getDefaultValue();
  }

  /**
   * Legt fest, ob die Komponente nur lesbar ist (ihr Wert nicht verändert
   * werden kann).
   *
   * @param boolean $readOnly true: Wert nur lesbar, false: Wert veränderbar
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setReadOnly($readOnly)
  {
    $this->setWidgetProperty('readOnly', $readOnly ? true : false);

    return $this;
  }

  /**
   * Liefert die Einstellung, ob die Komponente nur lesbar ist, oder ob ihr
   * Wert verändert werden kann.
   *
   * @return boolean
   */
  public function getReadOnly()
  {
    return $this->getWidgetProperty('readOnly') ? true : false;
  }

  /**
   * Deaktiviert oder aktiviert die Komponente.
   *
   * In der Voreinstellung sind alle Komponenten aktiv.
   *
   * @param boolean $disabled true: aktivieren, false: deaktivieren
   * @return GhGuiFormComponent liefert die Komponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setDisabled($disabled)
  {
    $this->setWidgetProperty('disabled', $disabled ? true : false);

    return $this;
  }

  /**
   * Liefert die Einstellung, ob die Komponente deaktiviert ist.
   *
   * @return boolean
   */
  public function getDisabled()
  {
    return $this->getWidgetProperty('disabled') ? true : false;
  }

  /**
   * Liefert den Formular-Container dieser Komponente.
   * 
   * @return GhGuiForm
   */
  public function getForm()
  {
    $parent = $this->getParentContainer();
    while($parent)
    {
      if($parent instanceof GhGuiForm)
        return $parent;
      $parent = $parent->getParentContainer();
    }

    return null;
  }

  /**
   * Initialisiert die Komponente für das Abrufen des HTML-Codes.
   */
  protected function _initHtml()
  {
    if(!parent::_initHtml())
      return false;

    if($this->getForm() === null)
      throw new GhGuiException(get_class($this) . ' has no parent GhGuiForm container');

    $value = $this->getCurrentValue();
    if($value !== null)
      $this->setWidgetProperty('value', $value);

    return true;
  }

  /**
   * Liefert die Objekteigenschaften für das Widget.
   *
   * @return array
   */
  protected function _getWidgetCreationProperties()
  {
    $properties = parent::_getWidgetCreationProperties();

    if(!$this->_createNameAndValueWidgetProperties)
    {
      if(isset($properties['name']))
        unset($properties['name']);

      if(isset($properties['value']))
        unset($properties['value']);
    }

    return $properties;
  }

  /**
   * Liefert den Datentyp, den eine bestimmte Widget-Eigenschaft haben muss.
   *
   * @param string $property Eigenschaft
   * @return string
   */
  protected function __getWidgetPropertyType($property)
  {
    switch($property)
    {
      case 'disabled':
        return 'boolean';

      case 'name':
      case 'value':
      case 'type':
        return 'string';
    }

    return parent::__getWidgetPropertyType($property);
  }
}