<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Icon-Set.
 *
 * Icon-Sets enthalten eine Bilddatei mit ein oder mehreren Icons, sowie eine
 * Liste der darauf abgebildeten Icons, in der die Koordinaten innerhalb der
 * Bilddatei zu jedem Icon-Namen enthalten sind.
 *
 * Optional kann für Hover-/Mouseover-Icons und für deaktivierte Icons eine
 * zusätzliche Bilddatei (mit denselben Koordinaten für die Icons) angeben
 * werden, oder eine Koordinate in der Bilddatei angegeben werden, ab der
 * die Hover-/Mouseover-Icons bzw. deaktivierten Icons beginnen. Die
 * Koordinaten der Icons müssen dann relativ ab diesem Punkt übereinstimmen.
 *
 * @package GhGui
 * @subpackage core
 * @category PHP-GUI-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiIconSet
{
  /**
   * URL der Icon-Bilddatei.
   * @var string
   */
  protected $_iconsUrl;

  /**
   * URL der Icon-Bilddatei für Hover-/Mouseover-Icons.
   * @var string
   */
  protected $_iconsHoverUrl;

  /**
   * X- und Y-Koordinate der Hover-/Mouseover-Icons.
   * Dieser Wert wird nur beachtet, wenn keine Bilddatei URL für
   * Hover-/Mouseover Icons gesetzt wurde.
   * @var array
   */
  protected $_hoverOffset;

  /**
   * URL der Icon-Bilddatei für deaktivierte Icons.
   * @var string
   */
  protected $_iconsDisabledUrl;

  /**
   * X- und Y-Koordinate der deaktivierten Icons.
   * Dieser Wert wird nur beachtet, wenn keine Bilddatei URL für deaktivierte
   * Icons gesetzt wurde.
   * @var array
   */
  protected $_disabledOffset;

  /**
   * Icon-Standardgröße.
   * Icons, für die keine explizite Größe angegeben ist verwenden diese
   * Standardgröße.
   * @var array
   */
  protected $_defaultSize = array(16, 16);

  /**
   * Icons und ihre Koordinaten.
   * Die Schlüssel des Arrays sind die Icon Namen und die Werte sind Arrays mit
   * X- und Y-Koordinate in der Bilddatei.
   * @var array
   */
  protected $_icons = array();

  /**
   * CSS Klasse des Icon-Sets.
   * @var string
   */
  protected $_iconClass;

  /**
   * Konstruktor.
   *
   * Es kann optional die URL der Bilddatei, in der die Icons enthalten sind,
   * angegeben werden (siehe setIconsUrl()), sowie eine Liste der Icons (siehe
   * addIcons()).
   *
   * @param string $iconsUrl URL der Bilddatei für die Icons
   * @param array  $icons    Liste der Icons
   */
  public function __construct($iconsUrl = null, $icons = null)
  {
    $this->_iconClass = 'icons_' . GhGui::createId();

    if($iconsUrl !== null)
      $this->setIconsUrl($iconsUrl);

    if($icons !== null)
      $this->addIcons($icons);
  }

  /**
   * Liefer die Stylesheet-Datei dieses Icon-Sets.
   *
   * Diese Methode ist standardmäßig leer. Sie kann überladen werden um die URL
   * einer CSS Stylsheet Datei zu liefern anstatt die CSS Angaben anhand der
   * Icon-Größen zu generieren.
   *
   * In diesem Fall brauchen weder die URLs der Bilddateien gesetzt werden,
   * noch die Icon-Größen und -Namen über die setIcons() Methode gesetzt
   * werden, da alle Icon-Style-Klassen in der CSS Stylesheet Datei enthalten
   * sind. Die Methode getIconSetClass() muss in diesem Fall ebenfalls überladen
   * werden um die Icon-Set CSS Klasse aus der Stylesheet Datei zu liefern.
   *
   * @return string
   */
  public function getStylesheetUrl()
  {
    return null;
  }

  /**
   * Liefert den CSS Klassennamen dieses Icon-Sets.
   *
   * @return string
   */
  public function getIconSetClass()
  {
    return $this->_iconClass;
  }

  /**
   * Setzt die URL für die Bilddatei in der die Icons enthalten sind.
   *
   * @param string $url URL der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconsUrl($url)
  {
    $this->_iconsUrl = $url;

    return $this;
  }

  /**
   * Liefert die URL der Bilddatei für die Icons.
   *
   * @return string
   */
  public function getIconsUrl()
  {
    return $this->_iconsUrl;
  }

  /**
   * Setzt die URL für die Bilddatei in der die Hover-/Mouseover-Icons
   * enthalten sind.
   *
   * @param string $url URL der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconsHoverUrl($url)
  {
    $this->_iconsHoverUrl = $url;

    return $this;
  }

  /**
   * Liefert die URL der Bilddatei für die Hover-/Mouseover-Icons.
   *
   * @return string
   */
  public function getIconsHoverUrl()
  {
    return $this->_iconsHoverUrl;
  }

  /**
   * Setzt die Koordinaten der Hover-/Mouseover-Icons innerhalb der Bilddatei.
   *
   * Wenn kein separates Bild für die Hover-/Mouseover-Icons vorliegt, sondern
   * diese in dem Bild der normalen Icons mit enthalten sind, kann hierüber der
   * Punkt gesetzt werden, ab dem die Hover-/Mouseover-Icons in dieser
   * Bilddatei beginnen. Die Icons müssen ab diesem Punkt dieselben Koordinaten
   * haben wie die normalen Icons.
   *
   * Wenn eine eigene Bilddatei für die Hover-/Mouseover-Icons gesetzt wurde
   * (über die Methode setIconsHoverUrl()), so wird dieser Wert ignoriert.
   *
   * @param int $left linker Rand der Hover-/Mouseover-Icons in der Bilddatei
   * @param int $top  oberer Rand der Hover-/Mouseover-Icons in der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconsHoverOffset($left, $top)
  {
    $this->_hoverOffset = array($left, $top);

    return $this;
  }

  /**
   * Liefert die Koordinaten der Hover-/Mouseover-Icons innerhalb der
   * Bilddatei.
   *
   * Der erste Wert im Array ist die X-Koordinate (linker Rand) und der zweite
   * Wert die Y-Koordinate (oberer Rand).
   *
   * @return array
   */
  public function getIconsHoverOffset()
  {
    return $this->_hoverOffset;
  }

  /**
   * Setzt die URL für die Bilddatei in der die deaktivierten Icons enthalten sind.
   *
   * @param string $url URL der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconsDisabledUrl($url)
  {
    $this->_iconsDisabledUrl = $url;

    return $this;
  }

  /**
   * Liefert die URL der Bilddatei für die deaktivierten Icons.
   *
   * @return string
   */
  public function getIconsDisabledUrl()
  {
    return $this->_iconsDisabledUrl;
  }

  /**
   * Setzt die Koordinaten der deaktivierten Icons innerhalb der Bilddatei.
   *
   * Wenn kein separates Bild für die deaktivierten Icons vorliegt, sondern
   * diese in dem Bild der normalen Icons mit enthalten sind, kann hierüber der
   * Punkt gesetzt werden, ab dem die deaktivierten Icons in dieser Bilddatei
   * beginnen. Die Icons müssen ab diesem Punkt dieselben Koordinaten haben wie
   * die normalen Icons.
   *
   * Wenn eine eigene Bilddatei für die deaktivierten Icons gesetzt wurde
   * (über die Methode setIconsDisabledUrl()), so wird dieser Wert ignoriert.
   *
   * @param int $left linker Rand der deaktivierten Icons in der Bilddatei
   * @param int $top  oberer Rand der deaktivierten Icons in der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setIconsDisabledOffset($left, $top)
  {
    $this->_disabledOffset = array($left, $top);

    return $this;
  }

  /**
   * Liefert die Koordinaten der deaktivierten Icons innerhalb der Bilddatei.
   *
   * Der erste Wert im Array ist die X-Koordinate (linker Rand) und der zweite
   * Wert die Y-Koordinate (oberer Rand).
   *
   * @return array
   */
  public function getIconsDisabledOffset()
  {
    return $this->_disabledOffset;
  }

  /**
   * Setzt die Standardgröße für Icons, für die keine explizite Größe
   * festgelegt wird.
   *
   * @param int $width  Standardbreite in Pixeln
   * @param int $height Standardhöhe in Pixeln
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setDefaultIconSize($width, $height)
  {
    $this->_defaultSize = array($width, $height);

    return $this;
  }

  /**
   * Liefert die Standardgröße für Icons, für die keine explizite Größe
   * festgelegt wird.
   *
   * Der erste Wert im Array ist die Breite und der zweite die Höhe in Pixeln.
   *
   * @return array
   */
  public function getDefaultIconSize()
  {
    return $this->_defaultSize;
  }

  /**
   * Fügt ein Icon zu Liste der enthaltenen Icons hinzu.
   *
   * @param string $name Name des Icons
   * @param int    $left X-Koordinate (linker Rand) in der Bilddatei
   * @param int    $top  Y-Koordinate (oberer Rand) in der Bilddatei
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addIcon($name, $left, $top)
  {
    $this->_icons[$name] = array($left, $top);

    return $this;
  }

  /**
   * Fügt mehrere Icons zur Liste der enthaltenen Icons hinzu.
   *
   * Die Schlüssel des übergebenen Arrays sind die Namen der Icons, die Werte
   * sind Arrays mit mindestens zwei Werten.
   * Der erste Wert ist die X-Koordinate (linker Rand) und der zweite die
   * Y-Koordinate (oberer Rand) in der Bilddatei. Sind weitere Werte enthalten,
   * so ist der dritte Wert die Breite des Icons und der vierte Wert die Höhe
   * des Icons in Pixeln. Werden Breite und Höhe nicht angegeben, so verwendet
   * dieses Icon die Standardgröße dieses Icon-Sets.
   *
   * @param array $icons Icons
   * @return GhGuiIconSet liefert das Icon-Set zurück um verkettete Aufrufe zu ermöglichen
   */
  public function addIcons($icons)
  {
    foreach($icons as $name => $offset)
      $this->addIcon($name, $offset[0], $offset[1]);

    return $this;
  }

  /**
   * Liefert die Liste der Icons dieses Icon-Sets.
   *
   * @return array
   */
  public function getIcons()
  {
    return $this->_icons;
  }

  /**
   * Liefert die Position und ggf. die Größe eines Icons innerhalb der
   * Bilddatei.
   *
   * @param string $icon Name des Icons
   * @return array
   */
  public function getIcon($icon)
  {
    if(isset($this->_icons[$icon]))
      return $this->_icons[$icon];
    else
      return null;
  }

  /**
   * Liefert den CSS Code für dieses Icon-Set.
   *
   * @return string
   */
  public function getCss()
  {
    $iconClass = $this->getIconSetClass();
    $iconsUrl = $this->getIconsUrl();
    $iconsHoverUrl = $this->getIconsHoverUrl();
    if($iconsHoverUrl)
      $iconsHoverOffset = null;
    else
      $iconsHoverOffset = $this->getIconsHoverOffset();
    $iconsDisabledUrl = $this->getIconsDisabledUrl();
    if($iconsDisabledUrl)
      $iconsDisabledOffset = null;
    else
      $iconsDisabledOffset = $this->getIconsDisabledOffset();
    $size = $this->getDefaultIconSize();

    $css = "\r\n";

    $css .= '.' . $iconClass . ' {' . GhGui::$eol;
    $css .= 'background:url("' . GhCoreStrings::attributeValue(GhGui::createUrl($iconsUrl)) . '") no-repeat;' . GhGui::$eol;
    $css .= 'width:' . $size[0] . 'px;' . GhGui::$eol;
    $css .= 'height:' . $size[1] . 'px;' . GhGui::$eol;
    $css .= 'text-align:center;' . GhGui::$eol;
    $css .= '}' . "\r\n";

    if($iconsHoverUrl)
    {
      $css .= '.dijitHover .' . $iconClass . ' {' . GhGui::$eol;
      $css .= 'background-image:url("' . GhCoreStrings::attributeValue(GhGui::createUrl($iconsHoverUrl)) . '");' . GhGui::$eol;
      $css .= '}' . "\r\n";
    }

    if($iconsDisabledUrl)
    {
      $css .= '.dijitDisabled .' . $iconClass . ' {' . GhGui::$eol;
      $css .= 'background-image:url("' . GhCoreStrings::attributeValue(GhGui::createUrl($iconsDisabledUrl)) . '");' . GhGui::$eol;
      $css .= '}' . "\r\n";
    }

    foreach($this->getIcons() as $name => $offset)
    {
      $css .= '.' . $iconClass . '.' . $name . ' {' . GhGui::$eol;
      $css .= 'background-position:-' . $offset[0] . 'px -' . $offset[1] . 'px;' . GhGui::$eol;
      if(isset($offset[2]))
        $css .= 'width:' . $offset[2] . 'px;' . GhGui::$eol;
      if(isset($offset[3]))
        $css .= 'height:' . $offset[3] . 'px;' . GhGui::$eol;
      $css .= '}' . "\r\n";

      if($iconsHoverOffset)
      {
        $css .= '.dijitHover .' . $iconClass . '.' . $name . ' {' . GhGui::$eol;
        $css .= 'background-position:-' . ($iconsHoverOffset[0] + $offset[0]) . 'px -' . ($iconsHoverOffset[1] + $offset[1]) . 'px;' . GhGui::$eol;
        $css .= '}' . "\r\n";
      }

      if($iconsDisabledOffset)
      {
        $css .= '.dijitDisabled .' . $iconClass . '.' . $name . ' {' . GhGui::$eol;
        $css .= 'background-position:-' . ($iconsDisabledOffset[0] + $offset[0]) . 'px -' . ($iconsDisabledOffset[1] + $offset[1]) . 'px;' . GhGui::$eol;
        $css .= '}' . "\r\n";
      }
    }

    return $css;
  }
}
