<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Formatierer für Datums- und Zeitformate.
 * 
 * @package GhGui
 * @subpackage formatter
 * @category PHP-GUI-Formatierer
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiFormatterDate extends GhGuiFormatter
{
  /**
   * Internationales Datumsformat (Jahr-Monat-Tag).
   */
  const DATE_ISO = 'date-iso';

  /**
   * Deutsches Datumsformat (Tag.Monat.Jahr).
   */
  const DATE_GERMAN = 'date-de';

  /**
   * Internationales Datums- und Zeitformat (Jahr-Monat-Tag Stunde:Minute:Sekunde).
   */
  const DATETIME_ISO = 'datetime-iso';

  /**
   * Deutsches Datums- und Zeitformat (Tag.Monat.Jahr Stunde:Minute:Sekunde).
   */
  const DATETIME_GERMAN = 'datetime-de';

  /**
   * Internationales Zeitformat (Stunde:Minute:Sekunde).
   */
  const TIME_ISO = 'time-iso';

  /**
   * Deutsches Zeitformat (Stunde:Minute:Sekunde).
   */
  const TIME_GERMAN = 'time-de';

  /**
   * Datums-/Zeitformat.
   * @var string
   */
  protected $_format = self::DATETIME_ISO;

  /**
   * Konstruktor.
   *
   * Optional wird das Datums-/Zeitformat in Form einer der Konstanten
   * angegeben:
   *
   * <dl>
   * <dt>GhGuiFormatterDate::DATE_ISO</dt>
   * <dd>Datum im ISO-Format (yyyy-mm-dd)</dd>
   * <dt>GhGuiFormatterDate::DATETIME_ISO</dt>
   * <dd>Datum und Zeit im ISO-Format (yyyy-mm-dd hh:mm:ss)</dd>
   * <dt>GhGuiFormatterDate::TIME_ISO</dt>
   * <dd>Zeit im ISO-Format (hh:mm:ss)</dd>
   * <dt>GhGuiFormatterDate::DATE_GERMAN</dt>
   * <dd>Datum im deutschen Format (dd.mm.yyyy)</dd>
   * <dt>GhGuiFormatterDate::DATETIME_GERMAN</dt>
   * <dd>Datum und Zeit im deutschen Format (dd.mm.yyyy hh:mm:ss)</dd>
   * <dt>GhGuiFormatterDate::TIME_GERMAN</dt>
   * <dd>Zeit im deutschen Format (hh:mm:ss)</dd>
   * </dl>
   *
   * Die Voreinstellung ist GhGuiFormatterDate::DATETIME_ISO.
   *
   * @param string $format Datums-/Zeitformat
   */
  public function __construct($format = null)
  {
    parent::__construct();

    if($format !== null)
      $this->_format = $format;
  }

  /**
   * Liefert den Javascript Code zum Registrieren dieses Formatierers.
   *
   * @param GhGuiComponent $source Komponente, auf der der Formatierer registriert werden soll
   * @return string
   */
  public function getJavascript(GhGuiComponent $source)
  {
    $javascript = 'function(data){var d=data;if(!(d instanceof Date))d=new Date(data);if(d.getTime()==0)return "";var v="";';

    if($this->_format == self::DATE_ISO || $this->_format == self::DATETIME_ISO)
      $javascript .= 'v+=d.getFullYear()+"-";var month=d.getMonth()+1;if(month<10)v+="0";v+=month+"-";var day=d.getDate();if(day<10)v+="0";v+=day;';
    elseif($this->_format == self::DATE_GERMAN || $this->_format == self::DATETIME_GERMAN)
      $javascript .= 'var day=d.getDate();if(day<10)v+="0";v+=day+".";var month=d.getMonth()+1;if(month<10)v+="0";v+=month+".";v+=d.getFullYear();';

    if($this->_format == self::DATETIME_ISO || $this->_format == self::DATETIME_GERMAN)
      $javascript .= 'v+=" ";';

    if($this->_format == self::TIME_ISO || $this->_format == self::DATETIME_ISO || $this->_format == self::TIME_GERMAN || $this->_format == self::DATETIME_GERMAN)
      $javascript .= 'var hours=d.getHours();if(hours<10)v+="0";v+=hours+":";var minutes=d.getMinutes();if(minutes<10)v+="0";v+=minutes+":";var seconds=d.getSeconds();if(seconds<10)v+="0";v+=seconds;';

    $javascript .= 'return v;}';

    return $javascript;
  }
}