<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Webseitenkomponente für Client Erweiterungen.
 *
 * Diese Webseite verwendet dieselbe Hintergrundfarbe wie der GREYHOUND Client
 * und stellt die Komponenten im Stil des Clients dar um eine möglichst
 * nahtlose visuelle Integration in den Client zu ermöglichen.
 *
 * Styles und Attribute, die auf der Webseitenkomponente gesetzt werden,
 * werden auf den 'body' Knoten der Webseite angewendet.
 *
 * @package GhGui
 * @subpackage page
 * @category PHP-GUI-Webseitenkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiExtensionPage extends GhGuiPage
{
  /**
   * CSS Klassen für den Haupt-DOM-Knoten dieser Komponente.
   * @var array
   */
  protected $_classes = array('greyhound');

  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * ID des DOM-Knotens, der alle Seitenkomponenten enthält.
   * @var string
   */
  protected $_pageWrapperId;

  /**
   * Komponenten dieser Seite.
   * @var array
   */
  protected $_components = array();

  /**
   * Konstruktor.
   *
   * Optional kann der Titel der Webseite (für den 'title' Knoten) und die
   * Basis-URL der Webseite angegeben werden.
   *
   * @param string $title   Titel der Webseite
   * @param string $baseUrl Basis-URL der Webseite
   */
  public function __construct($title = null, $baseUrl = null)
  {
    parent::__construct($title, $baseUrl);

    $this->_stylesheets[] = 'ghgui://lib/greyhound/page/ghguiextensionpage/screen.css';
    $this->_pageWrapperId = GhGui::createId();
  }

  /**
   * Fügt eine Komponente zum Inhalt der Webseite hinzu.
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addComponent($component)
  {
    $component->_setParentContainer($this);
    $this->_components[] = $component;

    return $component;
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    return array_merge($this->_components, $this->_globalComponents);
  }

  /**
   * Liefert den Javascript Code zum Erzeugen der Komponenten, in denen alle
   * Komponenten der Webseite enthalten sind.
   *
   * @return string
   */
  protected function _getHeadCreatePageComponentsJavascript()
  {
    return 'var widget_' . $this->_pageWrapperId . ' = new dijit.layout.ContentPane({style:"width:100%;height:100%;border:0px;margin:0px;padding:0px;"},"' . $this->_pageWrapperId . '");' . GhGui::$eol;
  }

  /**
   * Liefert den Javascript Code zum Initialisieren bzw. Anzeigen der
   * Komponenten, in denen alle Komponenten der Webseite enthalten sind.
   *
   * @return string
   */
  protected function _getHeadInitPageComponentsJavascript()
  {
    return 'widget_' . $this->_pageWrapperId . '.startup();' . GhGui::$eol;
  }

  /**
   * Liefert den vollständigen Webseiten-Code.
   *
   * @return string
   */
  protected function _getHtml()
  {
    // Seitenanfang:
    $html = GhGui::createHtmlStart();

    // 'head' Knoten:
    $html .= '<head>' . GhGui::$eol;

    foreach ($this->getHead() as $header)
      $html .= $header;

    $html .= GhGui::$eol . '</head>' . GhGui::$eol;

    // Seiteninhalt mit einem Splash-Screen überlagern, der verborgen wird, sobald die Seite aufgebaut ist:
    $body = $this->_getSplashScreenHtml();
    // Komponenten:
    $components = '';
    foreach ($this->_getAllComponents() as $component)/* @var GhGuiComponent $component */
      $components .= $component->_getHtml();
    // Alle Seitenelemente in ein div packen, auf diesem wird später startup() aufgerufen um das Layout zu initialisieren:
    $body .= GhGui::createHtml('div', array('id' => $this->_pageWrapperId), $components);
    // Body in HTML einfügen:
    $html .= GhGui::createHtml('body', $this->_getAllAttributes(), $body);

    // Seitenende:
    $html .= GhGui::createHtmlEnd();

    return $html;
  }
}