<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Webseitenkomponente für Seiten im Stil der GREYHOUND Webseite.
 *
 * @package GhGui
 * @subpackage page
 * @category PHP-GUI-Webseitenkomponenten
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhGuiWebPage extends GhGuiPage
{
  /**
   * Die von dieser Komponente benötigten Widget Klassen.
   * @var array
   */
  protected $_requiredWidgetClasses = array('dijit.layout.ContentPane');

  /**
   * Titel für die Startseite.
   * Dieser Titel wird als erstes Element des Seitenpfades (Breadcrumb) und als
   * Titel neben dem Logo verwendet.
   * @var string
   */
  protected $_homeTitle;

  /**
   * URL zur Startseite.
   * @var string
   */
  protected $_homeUrl;

  /**
   * Obere Seitennavigation (Hauptmenu).
   * @var GhCoreTree
   */
  protected $_topNavigation;

  /**
   * Seitennavigation im Fußbereich.
   * @var GhCoreTree
   */
  protected $_footerNavigation;

  /**
   * Inhalt (Komponenten) des Fußbereichs.
   * @var array
   */
  protected $_footer = array();

  /**
   * Breadcrumb-Navigation (Webseiten-Pfad).
   * @var GhCoreTree
   */
  protected $_breadCrumbs;

  /**
   * Baumknoten-ID ab der die Breadcrumb-Navigation beginnt.
   * @var int
   */
  protected $_breadCrumbsId = 0;

  /**
   * Komponenten des Infobereichs (grauer Bereich unter dem Seitenkopf und über
   * dem Seiteninhalt).
   * @var array
   */
  protected $_info = array();

  /**
   * Seiteninhalt (Komponenten).
   * @var array
   */
  protected $_content = array();

  /**
   * ID des DOM-Knotens, der alle Seitenkomponenten enthält.
   * @var string
   */
  protected $_pageWrapperId;

  /**
   * ID des DOM-Knotens, der die Seiteninhalt-Komponenten enthält.
   * @var string
   */
  protected $_contentWrapperId;

  /**
   * Höhe des Inhaltsbereichs der Webseite.
   * @var string
   */
  protected $_contentHeight = '800px';

  /**
   * Konstruktor.
   *
   * Optional kann der Titel der Webseite (für den 'title' Knoten) und die
   * Basis-URL der Webseite angegeben werden.
   *
   * @param string $title   Titel der Webseite
   * @param string $baseUrl Basis-URL der Webseite
   */
  public function __construct($title = null, $baseUrl = null)
  {
    parent::__construct($title, $baseUrl);

    $this->_splashScreenStyles['background'] = '#000';
    $this->_stylesheets[] = 'ghgui://lib/greyhound/page/ghguiwebpage/screen.css';

    $this->_pageWrapperId = GhGui::createId();
    $this->_contentWrapperId = GhGui::createId();
  }

  /**
   * Setzt den Titel für die Startseite.
   * Dieser Titel wird als erstes Element des Seitenpfades (Breadcrumb) und als
   * Titel neben dem Logo verwendet.
   *
   * @param string $homeTitle Titel für die Startseite
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setHomeTitle($homeTitle)
  {
    $this->_homeTitle = $homeTitle;

    return $this;
  }

  /**
   * Liefert den Titel für die Startseite.
   * @return string
   */
  public function getHomeTitle()
  {
    return $this->_homeTitle;
  }

  /**
   * Setzt die URL zur Startseite.
   *
   * Diese URL wird für das erste Element des Seitenpfades (Breadcrumb)
   * verwendet.
   *
   * @param string $homeUrl URL zur Startseite
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setHomeUrl($homeUrl)
  {
    $this->_homeUrl = $homeUrl;

    return $this;
  }

  /**
   * Liefert die URL der Startseite.
   *
   * @return string
   */
  public function getHomeUrl()
  {
    return $this->_homeUrl;
  }

  /**
   * Fügt eine Komponente zum Inhalt der Webseite hinzu.
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addContentComponent($component)
  {
    $component->_setParentContainer($this);
    $this->_content[] = $component;

    return $component;
  }

  /**
   * Liefert die Inhaltskomponenten.
   *
   * @return array
   */
  public function getContentComponents()
  {
    return $this->_content;
  }

  /**
   * Setzt die obere Seitennavigation (das Hauptmenu).
   *
   * @param GhCoreTree $tree Navigationshierarchie
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setTopNavigation(GhCoreTree $tree)
  {
    $this->_topNavigation = $tree;

    return $this;
  }

  /**
   * Liefert die obere Seitennavigation (das Hauptmenu).
   *
   * @return GhCoreTree
   */
  public function getTopNavigation()
  {
    if(!$this->_topNavigation)
      $this->_topNavigation = new GhCoreTree();

    return $this->_topNavigation;
  }

  /**
   * Setzt die untere Seitennavigation (im Fußbereich).
   *
   * @param GhCoreTree $tree Navigationshierarchie
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setFooterNavigation(GhCoreTree $tree)
  {
    $this->_footerNavigation = $tree;

    return $this;
  }

  /**
   * Liefert die untere Seitennavigation (im Fußbereich).
   *
   * @return GhCoreTree
   */
  public function getFooterNavigation()
  {
    if(!$this->_footerNavigation)
      $this->_footerNavigation = new GhCoreTree();

    return $this->_footerNavigation;
  }

  /**
   * Fügt eine Komponente zum Fußbereich hinzu.
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addFooterComponent(GhGuiComponent $component)
  {
    $component->_setParentContainer($this);
    $this->_footer[] = $component;

    return $component;
  }

  /**
   * Liefert die Komponenten des Fußbereichs zurück.
   *
   * @return array
   */
  public function getFooterComponents()
  {
    return $this->_footer;
  }

  /**
   * Setzt die Breadcrumb-Navigation (den Seitenpfad).
   *
   * @param GhCoreTree $tree Navigationshierarchie
   * @param int        $id   ID des Baumknotens, der die aktive Webseite
   *  repräsentiert (der Endpunkt der Breadcrumb-Navigation)
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setBreadCrumbs(GhCoreTree $tree, $id)
  {
    $this->_breadCrumbs = $tree;
    $this->_breadCrumbsId = $id;

    return $this;
  }

  /**
   * Liefert die Breadcrumb-Navigation (den Seitenpfad).
   *
   * @param int $id ID des Baumknotens der die aktive Webseite repräsentiert
   *   (der Endpunkt der Breadcrumb-Navigation)
   * @return GhCoreTree
   */
  public function getBreadCrumbs($id = null)
  {
    if(!$this->_breadCrumbs)
      $this->_breadCrumbs = new GhCoreTree();

    if($id !== null)
      $this->_breadCrumbsId = $id;

    return $this->_breadCrumbs;
  }

  /**
   * Fügt eine Komponente in den Infobereich der Webseite ein.
   *
   * Der Infobereich wird als hellgrauer Bereich unter dem Seitenkopf und
   * über dem Seiteninhalt angezeigt (wenn er Komponenten enthält).
   *
   * @param GhGuiComponent $component Komponente
   * @return GhGuiComponent liefert die hinzugefügte Komponente zurück
   */
  public function addInfoComponent(GhGuiComponent $component)
  {
    $component->_setParentContainer($this);
    $this->_info[] = $component;

    return $component;
  }

  /**
   * Liefert die Komponenten des Infobereichs der Webseite.
   *
   * @return array
   */
  public function getInfoComponents()
  {
    return $this->_info;
  }

  /**
   * Setzt die Höhe des Inhaltsbereichs.
   *
   * @param string $height Höhe (CSS Style Angabe)
   * @return GhGuiWebPage liefert die Webseitenkomponente zurück um verkettete Aufrufe zu ermöglichen
   */
  public function setContentHeight($height)
  {
    $this->_contentHeight = $height;

    return $this;
  }

  /**
   * Liefert die Höhe des Inhaltsbereichs.
   *
   * @return string
   */
  public function getContentHeight()
  {
    return $this->_contentHeight;
  }

  /**
   * Liefert einen Baum als unsortierte HTML-Liste.
   *
   * Falls die Daten der Baumknoten Arrays sind und einen Schlüssel 'url'
   * besitzen, dann wird der Baumknoten als Hyperlink dargestellt und der Wert
   * dieses Schlüssels als URL für den Hyperlink verwendet.
   *
   * @param GhCoreTreeBase $treeNode
   * @return string
   */
  protected function _getTreeHtml(GhCoreTreeBase $treeNode)
  {
    if(!count($treeNode->getChildren()))
      return '';

    $children = $treeNode->getChildren();

    $html = '';
    for($i = 0, $n = count($children); $i < $n; $i++)
    {
      $url = $this->_getTreeNodeUrl($children[$i]);
      if($url)
        $node = GhGui::createHtml('a', array('href' => $url), $children[$i]->getCaption());
      else
        $node = $children[$i]->getCaption();
      $html .= GhGui::createHtml('li', null, $node . $this->_getTreeHtml($children[$i]));
    }

    $html = GhGui::createHtml('ul', null, $html);

    return $html;
  }

  /**
   * Liefert die URL eines Baumknotens für die Navigation.
   *
   * Falls die Daten des Baumknotens ein Array mit einem Schlüssel 'url' sind,
   * dann wird dieser Wert als URL geliefert. Ansonsten wird ein leerer String
   * geliefert.
   *
   * @param GhCoreTreeBase $treeNode Baumknoten
   * @return string
   */
  protected function _getTreeNodeUrl(GhCoreTreeBase $treeNode)
  {
    $data = $treeNode->getData();
    return isset($data['url']) ? $data['url'] : '';
  }

  /**
   * Liefert die Beschriftung eines Baumknotens für die Navigation.
   *
   * @param GhCoreTreeBase $treeNode Baumknoten
   * @return string
   */
  protected function _getTreeNodeCaption(GhCoreTreeBase $treeNode)
  {
    return $treeNode->getCaption();
  }

  /**
   * Liefert den HTML Code für den Navigations-Link eines Baumknotens.
   *
   * @param GhCoreTreeBase $treeNode Baumknoten
   * @return string
   */
  protected function _getTreeNodeHtml(GhCoreTreeBase $treeNode)
  {
    $captionHtml = GhGui::createHtml('span', array('class' => 'ghnavlabel'), $this->_getTreeNodeCaption($treeNode));

    $iconHtml = '';
    $data = $treeNode->getData();
    if(isset($data['icon']))
      $iconHtml = GhGui::createHtml('span', array('class' => 'ghnavicon ' . GhCoreStrings::attributeValue($this->getIconSet()->getIconSetClass() . ' ' . $data['icon'])));
    elseif(isset($data['iconUrl']))
      $iconHtml = GhGui::createHtml('span', array('class' => 'ghnavicon', 'style' => 'background-image:url(\'' . $data['iconUrl'] . '\');'));

    $html = GhGui::createHtml('span', null, $iconHtml . $captionHtml);

    $html = GhGui::createHtml('a', array('href' => $this->_getTreeNodeUrl($treeNode)), $html);

    return $html;
  }

  /**
   * Liefert die Kopfzeile für den Seitentitel.
   *
   * @return string
   */
  protected function _getHeadTitle()
  {
    $title = $this->getTitle();
    if(!empty($title))
      $title .= ' &raquo; ';
    $title .= $this->_homeTitle;

    return GhGui::createHtml('title', null, $title);
  }

  /**
   * Liefert alle in diesem Container enthaltenen Komponenten.
   *
   * @return array
   */
  protected function _getAllComponents()
  {
    return array_merge($this->_info, $this->_content, $this->_footer, $this->_globalComponents);
  }

  /**
   * Liefert den Javascript Code zum Erzeugen der Komponenten, in denen alle
   * Komponenten der Webseite enthalten sind.
   *
   * @return string
   */
  protected function _getHeadCreatePageComponentsJavascript()
  {
    $javascript = '';

    $javascript .= 'var widget_' . $this->_pageWrapperId . '=new dijit.layout.ContentPane({style:"width:100%;height:100%;border:0px;margin:0px;padding:0px;"},"' . $this->_pageWrapperId . '");' . GhGui::$eol;

    $javascript .= 'var widget_' . $this->_contentWrapperId . '=new dijit.layout.ContentPane({style:"width:800px;height:' . $this->_contentHeight . ';"},"ghblock_html");' . GhGui::$eol;

    return $javascript;
  }

  /**
   * Liefert den Javascript Code zum Initialisieren bzw. Anzeigen der
   * Komponenten, in denen alle Komponenten der Webseite enthalten sind.
   *
   * @return string
   */
  protected function _getHeadInitPageComponentsJavascript()
  {
    return 'widget_' . $this->_pageWrapperId . '.startup();' . GhGui::$eol;
  }

  /**
   * Liefert den vollständigen Seiten-Code.
   * Diese Methode liefert den kompletten Webseiten-Code, inklusive DOCTYPE.
   *
   * @return string
   */
  protected function _getHtml()
  {
    // Seitenanfang:
    $html = GhGui::createHtmlStart();

    // 'head' Knoten:
    $html .= '<head>' . GhGui::$eol;

    foreach($this->getHead() as $header)
      $html .= $header;

    $html .= '</head>' . GhGui::$eol;

    $components = '';

    // Inhalt (für 'body' Knoten):
    $components .= '<div id="ghpage">' . GhGui::$eol;

    // Kopfbereich der Seite:
    $components .= '<div id="ghheader">' . GhGui::$eol;
    $components .= '<div id="ghtitle">' . $this->_homeTitle . '</div>' . GhGui::$eol;
    $components .= '<a id="ghlogo" href="' . $this->_homeUrl . '">' . GhGui::createHtml('br') . '</a>' . GhGui::$eol;
    $components .= '</div>' . GhGui::$eol;

    if(!empty($this->_topNavigation))
    {
      $components .= '<div id="ghnav">' . GhGui::$eol .
               ' <ul>' . GhGui::$eol;

      $children = $this->_topNavigation->getChildren();

      for($i = 0, $n = sizeof($children); $i < $n; $i++)
      {
        $components .= '  <li>' . GhGui::$eol .
                 '   <a href="' . $this->_getTreeNodeUrl($children[$i]) . '">' . $children[$i]->getCaption() . '</a>';
        $subchildren = $children[$i]->getChildren();

        if(sizeof($subchildren) > 0)
        {
          $components .= GhGui::$eol . '   <div' . ($i + 1 == $n ? ' class="last"' : '') . '>' . GhGui::$eol .
                   '    <ul>' . GhGui::$eol;

          for($j = 0, $m = sizeof($subchildren); $j < $m; $j++)
          {
            $components .= '     <li>' . $this->_getTreeNodeHtml($subchildren[$j]) . '</li>' . GhGui::$eol;
          }

          $components .= '    </ul>' . GhGui::$eol .
                   '   </div>' . GhGui::$eol .
                   '  </li>' . GhGui::$eol;
        }
        else
          $components .= '</li>' . GhGui::$eol;
      }

      $components .= ' </ul>' . GhGui::$eol .
               '</div>' . GhGui::$eol;
    }

    $components .= '<div id="ghshadow_left">' . GhGui::$eol;
    $components .= '<div id="ghshadow_right">' . GhGui::$eol;
    $components .= '<div id="ghcontentbox">' . GhGui::$eol;

    // Infobereich:
    if(!empty($this->_info))
    {
      $components .= '<div id="ghinfo">' . GhGui::$eol;

      foreach($this->_info as $component)
        $components .= $component->_getHtml();

      $components .= '</div>' . GhGui::$eol;
    }

    // Inhaltsbereich:
    $components .= '<div id="ghcontent">' . GhGui::$eol;
    $components .= '<div id="ghzipper">' . GhGui::createHtml('span') . '</div>' . GhGui::$eol;

    if(!empty($this->_breadCrumbs))
    {
      $treeNode = $this->_breadCrumbs->get($this->_breadCrumbsId);
      $items = array();

      while(!empty($treeNode) && method_exists($treeNode, 'getCaption'))
      {
        $items[] = '<a href="' . $this->_getTreeNodeUrl($treeNode) . '">' . htmlspecialchars($treeNode->getCaption(), ENT_NOQUOTES, GhGui::getCharset()) . '</a>';
        $treeNode = $treeNode->getParent();
      }

      $items[] = '<a href="' . $this->_homeUrl . '">' . htmlspecialchars($this->_homeTitle, ENT_NOQUOTES, GhGui::getCharset()) . '</a>';

      $components .= '<div id="ghbreadcrumbs">' . implode(' &raquo; ', array_reverse($items)) . '</div>' . GhGui::$eol;
    }

    $components .= '<div id="ghcontent_inner">' . GhGui::$eol;
    $components .= '<div id="ghblock_html">' . GhGui::$eol;

    foreach($this->_content as $component)
      $components .= $component->_getHtml();

    $components .= '</div>' . GhGui::$eol; //ghblock_html
    $components .= '</div>' . GhGui::$eol; //ghcontent_inner
    $components .= '</div>' . GhGui::$eol; //ghcontent
    $components .= '</div>' . GhGui::$eol; //ghcontentbox
    $components .= '</div>' . GhGui::$eol; //ghshadow_right
    $components .= '</div>' . GhGui::$eol; //ghshadow_left

    $components .= '</div>' . GhGui::$eol; //ghpage

    // footer:
    $components .= '<div id="ghfooter">' . GhGui::$eol;

    if(!empty($this->_footerNavigation))
    {
      $components .= '<ul>' . GhGui::$eol;

      $children = $this->_footerNavigation->getChildren();

      for($i = 0, $n = sizeof($children); $i < $n; $i++)
        $components .= '<li' . ($i == 0 ? ' class="first"' : '') . '><a href="' . $this->_getTreeNodeUrl($children[$i]) . '">' . $children[$i]->getCaption() . '</a></li>';

      $components .= '</ul>' . GhGui::$eol;
    }

    if(!empty($this->_footer))
    {
      $components .= '<div id="ghfooter_content">' . GhGui::$eol;

      foreach($this->_footer as $component)
        $components .= $component->_getHtml();

      $components .= '</div>' . GhGui::$eol;
    }

    $components .= '</div>' . GhGui::$eol;

    foreach($this->_globalComponents as $component)
      $components .= $component->_getHtml();

    // Seiteninhalt mit einem Splash-Screen überlagern, der verborgen wird, sobald die Seite aufgebaut ist:
    $body = $this->_getSplashScreenHtml();
    // Alle Seitenelemente in ein div packen, auf diesem wird später startup() aufgerufen um das Layout zu initialisieren:
    $body .= GhGui::createHtml('div', array('id' => $this->_pageWrapperId), $components);
    // Body in HTML einfügen:
    $html .= GhGui::createHtml('body', $this->_getAllAttributes(), $body);

    // Seitenende:
    $html .= GhGui::createHtmlEnd();

    return $html;
  }
}