<?php
/*
 * GREYHOUND PHP API
 *
 * Copyright (c) 2011-2021 digital guru GmbH & Co. KG
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of the GREYHOUND PHP API and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including without
 * limitation the rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Baum-Struktur für RPC-Objekte.
 *
 * @package GhRpc
 * @subpackage core
 * @category PHP-RPC-Kernklassen
 *
 * @author digital guru GmbH &amp; Co. KG <develop@greyhound-software.com>
 * @copyright 2011-2021 digital guru GmbH &amp; Co. KG
 * @link greyhound-software.com
 */
class GhRpcTree extends GhCoreTree
{
  /**
   * Konstruktor.
   * Anmerkung: Beim Erzeugen eines Baumes für RPC Item-Objekte (aus der rpc_itemslib) sollte 'Subject' als
   * $captionProperty angegeben werden, da die Item-Objekte nicht über eine Eigenschaft 'Name' verfügen.
   * Andere Objekt-Typen (z.B. Themen, Gruppen, Kennzeichnungen) haben eine Eigenschaft 'Name'.
   *
   * @param array   $list             Eine Liste von RPC-Objekten (z.B. aus einem Aufruf einer GetList RPC-Funktion)
   * @param string  $captionProperty  RPC-Objekt-Eigenschaft für den Knoten-Titel (Vorgabewert: 'Name')
   * @param string  $parentIdProperty RPC-Objekt-Eigenschaft für die ID des Eltern-Knotens (Vorgabewert: 'ParentRef')
   * @param boolean $rootId           ID des Elements, das als Wurzelknoten des Baums verwendet werden soll (es werden nur die Zweige unterhalb dieses Elements in den Baum eingefügt)
   */
  public function __construct($list = null, $captionProperty = 'Name', $parentIdProperty = 'ParentRef', $rootId = null)
  {
    parent::__construct();

    if($list)
      $this->addList($list, $captionProperty, $parentIdProperty, $rootId);
  }

  /**
   * Fügt RPC-Objekte dem Baum hinzu und ordnet sie entsprechend ihrer Eltern-ID in den Baum ein.
   *
   * @param array   $list             Eine Liste von RPC-Objekten (z.B. aus einem Aufruf einer GetList RPC-Funktion)
   * @param string  $captionProperty  RPC-Objekt-Eigenschaft für den Knoten-Titel (Vorgabewert: 'Name')
   * @param string  $parentIdProperty RPC-Objekt-Eigenschaft für die ID des Eltern-Knotens (Vorgabewert: 'ParentRef')
   * @param boolean $rootId           ID des Elements, das als Wurzelknoten des Baums verwendet werden soll (es werden nur die Zweige unterhalb dieses Elements in den Baum eingefügt)
   */
  public function addList($list, $captionProperty = 'Name', $parentIdProperty = 'ParentRef', $rootId = null)
  {
    $allowedParentIds = array();

    if($rootId)
      $allowedParentIds[] = $rootId;

    foreach($list as $item)
    {
      if($rootId && !in_array($item->$parentIdProperty, $allowedParentIds))
        continue;

      $caption = isset($item->$captionProperty) ? $item->$captionProperty : null;
      $parentId = isset($item->$parentIdProperty) ? $item->$parentIdProperty : 0;

      // Die Kinder des gewünschten Wurzelknotens sollen auf oberster Ebene eingefügt werden:
      if($rootId && $parentId == $rootId)
        $parentId = 0;

      $allowedParentIds[] = $item->ID;
      $this->addChild($item->ID, $parentId, $caption, $item);
    }
  }
}
